@@ -19,8 +19,7 @@
 
 import cairo
 import gettext
-from gi.repository import Gtk
-from gi.repository import GObject
+from gi.repository import Gtk, GObject
 import logging
 import os
 import xapian
@@ -37,8 +36,11 @@ from softwarecenter.ui.gtk3.models.appstore2 import AppPropertiesHelper
 from softwarecenter.ui.gtk3.widgets.viewport import Viewport
 from softwarecenter.ui.gtk3.widgets.containers import (
      FramedHeaderBox, FramedBox, FlowableGrid)
+from softwarecenter.ui.gtk3.widgets.recommendations import (
+                                        RecommendationsPanelLobby,
+                                        RecommendationsPanelCategory)
 from softwarecenter.ui.gtk3.widgets.exhibits import (
-    ExhibitBanner, FeaturedExhibit)
+                                        ExhibitBanner, FeaturedExhibit)
 from softwarecenter.ui.gtk3.widgets.buttons import (LabelTile,
                                                     CategoryTile,
                                                     FeaturedTile)
@@ -52,32 +54,35 @@ from softwarecenter.db.categories import (Category,
 from softwarecenter.db.utils import get_query_for_pkgnames
 from softwarecenter.distro import get_distro
 from softwarecenter.backend.scagent import SoftwareCenterAgent
+from softwarecenter.backend.reviews import get_review_loader
 
-LOG=logging.getLogger(__name__)
+LOG = logging.getLogger(__name__)
 
 
 _asset_cache = {}
+
+
 class CategoriesViewGtk(Viewport, CategoriesParser):
 
     __gsignals__ = {
-        "category-selected" : (GObject.SignalFlags.RUN_LAST,
-                               None, 
-                               (GObject.TYPE_PYOBJECT, ),
-                              ),
-                              
-        "application-selected" : (GObject.SignalFlags.RUN_LAST,
+        "category-selected": (GObject.SignalFlags.RUN_LAST,
+                              None,
+                              (GObject.TYPE_PYOBJECT, ),
+                             ),
+
+        "application-selected": (GObject.SignalFlags.RUN_LAST,
+                                 None,
+                                 (GObject.TYPE_PYOBJECT, ),
+                                ),
+
+        "application-activated": (GObject.SignalFlags.RUN_LAST,
                                   None,
                                   (GObject.TYPE_PYOBJECT, ),
                                  ),
 
-        "application-activated" : (GObject.SignalFlags.RUN_LAST,
-                                   None,
-                                   (GObject.TYPE_PYOBJECT, ),
-                                  ),
-                                  
-        "show-category-applist" : (GObject.SignalFlags.RUN_LAST,
-                                   None,
-                                   (),)
+        "show-category-applist": (GObject.SignalFlags.RUN_LAST,
+                                  None,
+                                  (),)
         }
 
     SPACING = PADDING = 3
@@ -86,17 +91,17 @@ class CategoriesViewGtk(Viewport, CategoriesParser):
     STIPPLE = os.path.join(softwarecenter.paths.datadir,
                            "ui/gtk3/art/stipple.png")
 
-    def __init__(self, 
+    def __init__(self,
                  datadir,
-                 desktopdir, 
+                 desktopdir,
                  cache,
                  db,
                  icons,
-                 apps_filter,
+                 apps_filter=None,  # FIXME: kill this, its not needed anymore?
                  apps_limit=0):
 
         """ init the widget, takes
-        
+
         datadir - the base directory of the app-store data
         desktopdir - the dir where the applications.menu file can be found
         db - a Database object
@@ -108,6 +113,8 @@ class CategoriesViewGtk(Viewport, CategoriesParser):
         self.cache = cache
         self.db = db
         self.icons = icons
+        self.properties_helper = AppPropertiesHelper(
+            self.db, self.cache, self.icons)
         self.section = None
 
         Viewport.__init__(self)
@@ -152,9 +159,9 @@ class CategoriesViewGtk(Viewport, CategoriesParser):
            amount = number of tiles to add from start of doc range'''
         amount = min(len(docs), amount)
         for doc in docs[0:amount]:
-            tile = FeaturedTile(self.helper, doc)
+            tile = FeaturedTile(self.properties_helper, doc)
             tile.connect('clicked', self.on_app_clicked,
-                         self.helper.get_application(doc))
+                         self.properties_helper.get_application(doc))
             flowgrid.add_child(tile)
         return
 
@@ -168,7 +175,8 @@ class CategoriesViewGtk(Viewport, CategoriesParser):
 
     def _cache_art_assets(self):
         global _asset_cache
-        if _asset_cache: return _asset_cache
+        if _asset_cache:
+            return _asset_cache
         assets = _asset_cache
         # cache the bg pattern
         surf = cairo.ImageSurface.create_from_png(self.STIPPLE)
@@ -185,7 +193,6 @@ class CategoriesViewGtk(Viewport, CategoriesParser):
             return False
 
         GObject.timeout_add(50, timeout_emit)
-        return
 
     def on_category_clicked(self, btn, cat):
         """emit the category-selected signal when a category was clicked"""
@@ -194,7 +201,6 @@ class CategoriesViewGtk(Viewport, CategoriesParser):
             return False
 
         GObject.timeout_add(50, timeout_emit)
-        return
 
     def build(self, desktopdir):
         pass
@@ -202,14 +208,14 @@ class CategoriesViewGtk(Viewport, CategoriesParser):
     def do_draw(self, cr):
         cr.set_source(_asset_cache["stipple"])
         cr.paint_with_alpha(0.5)
-        for child in self: self.propagate_draw(child, cr)
-        return
+        for child in self:
+            self.propagate_draw(child, cr)
 
     def set_section(self, section):
         self.section = section
 
     def refresh_apps(self):
-        raise NotImplemented
+        raise NotImplementedError
 
 
 class LobbyViewGtk(CategoriesViewGtk):
@@ -226,21 +232,29 @@ class LobbyViewGtk(CategoriesViewGtk):
         # ensure that on db-reopen we refresh the whats-new titles
         self.db.connect("reopen", self._on_db_reopen)
 
+        # ensure that updates to the stats are reflected in the UI
+        self.reviews_loader = get_review_loader(self.cache)
+        self.reviews_loader.connect(
+            "refresh-review-stats-finished", self._on_refresh_review_stats)
+
         self.build(desktopdir)
         return
 
     def _on_db_reopen(self, db):
-        self._update_new_content()
+        self._update_whats_new_content()
+
+    def _on_refresh_review_stats(self, reviews_loader, review_stats):
+        self._update_top_rated_content()
 
     def _build_homepage_view(self):
         # these methods add sections to the page
         # changing order of methods changes order that they appear in the page
-        #~ self._append_recommendations()
         self._append_banner_ads()
 
         self.top_hbox = Gtk.HBox(spacing=StockEms.SMALL)
         top_hbox_alignment = Gtk.Alignment()
-        top_hbox_alignment.set_padding(0, 0, StockEms.MEDIUM-2, StockEms.MEDIUM-2)
+        top_hbox_alignment.set_padding(0, 0, StockEms.MEDIUM - 2,
+            StockEms.MEDIUM - 2)
         top_hbox_alignment.add(self.top_hbox)
         self.vbox.pack_start(top_hbox_alignment, False, False, 0)
 
@@ -249,10 +263,9 @@ class LobbyViewGtk(CategoriesViewGtk):
         self.right_column = Gtk.Box.new(Gtk.Orientation.VERTICAL, self.SPACING)
         self.top_hbox.pack_start(self.right_column, True, True, 0)
 
-        self._append_new()
-        #~ self._append_recommendations()
+        self._append_whats_new()
         self._append_top_rated()
-
+        self._append_recommended_for_you()
         self._append_appcount()
 
         #self._append_video_clips()
@@ -261,33 +274,33 @@ class LobbyViewGtk(CategoriesViewGtk):
 
     #~ def _append_top_of_the_pops(self):
         #~ self.totp_hbox = Gtk.HBox(spacing=self.SPACING)
-#~ 
+#~
         #~ alignment = Gtk.Alignment()
         #~ alignment.set_padding(0, 0, self.PADDING, self.PADDING)
         #~ alignment.add(self.totp_hbox)
-#~ 
+#~
         #~ frame = FramedHeaderBox()
         #~ frame.header_implements_more_button()
         #~ frame.set_header_label(_("Most Popular"))
-#~ 
+#~
         #~ label = Gtk.Label.new("Soda pop!!!")
         #~ label.set_name("placeholder")
         #~ label.set_size_request(-1, 200)
-#~ 
+#~
         #~ frame.add(label)
         #~ self.totp_hbox.add(frame)
-#~ 
+#~
         #~ frame = FramedHeaderBox()
         #~ frame.header_implements_more_button()
         #~ frame.set_header_label(_("Top Rated"))
-#~ 
+#~
         #~ label = Gtk.Label.new("Demos ftw(?)")
         #~ label.set_name("placeholder")
         #~ label.set_size_request(-1, 200)
-#~ 
+#~
         #~ frame.add(label)
         #~ self.totp_hbox.add(frame)
-#~ 
+#~
         #~ self.vbox.pack_start(alignment, False, False, 0)
         #~ return
 
@@ -296,17 +309,17 @@ class LobbyViewGtk(CategoriesViewGtk):
         #~ frame.set_header_expand(False)
         #~ frame.set_header_position(HeaderPosition.LEFT)
         #~ frame.set_header_label(_("Latest Demo Videos"))
-#~ 
+#~
         #~ label = Gtk.Label.new("Videos go here")
         #~ label.set_name("placeholder")
         #~ label.set_size_request(-1, 200)
-#~ 
+#~
         #~ frame.add(label)
-#~ 
+#~
         #~ alignment = Gtk.Alignment()
         #~ alignment.set_padding(0, 0, self.PADDING, self.PADDING)
         #~ alignment.add(frame)
-#~ 
+#~
         #~ self.vbox.pack_start(alignment, False, False, 0)
         #~ return
 
@@ -333,18 +346,19 @@ class LobbyViewGtk(CategoriesViewGtk):
         # query using the agent
         scagent = SoftwareCenterAgent()
         scagent.connect(
-            "exhibits", lambda sca,l: exhibit_banner.set_exhibits(l))
+            "exhibits", lambda sca, l: exhibit_banner.set_exhibits(l))
         scagent.query_exhibits()
 
         a = Gtk.Alignment()
-        a.set_padding(0,StockEms.SMALL,0,0)
+        a.set_padding(0, StockEms.SMALL, 0, 0)
         a.add(exhibit_banner)
 
         self.vbox.pack_start(a, False, False, 0)
         return
 
     def _append_departments(self):
-        # set the departments section to use the label markup we have just defined
+        # set the departments section to use the label markup we have just
+        # defined
         cat_vbox = FramedBox(Gtk.Orientation.VERTICAL)
         self.top_hbox.pack_start(cat_vbox, False, False, 0)
 
@@ -353,7 +367,8 @@ class LobbyViewGtk(CategoriesViewGtk):
 
         mrkup = "<small>%s</small>"
         for cat in sorted_cats:
-            if 'carousel-only' in cat.flags: continue
+            if 'carousel-only' in cat.flags:
+                continue
             category_name = mrkup % GObject.markup_escape_text(cat.name)
             label = LabelTile(category_name, None)
             label.label.set_margin_left(StockEms.SMALL)
@@ -364,129 +379,88 @@ class LobbyViewGtk(CategoriesViewGtk):
             cat_vbox.pack_start(label, False, False, 0)
         return
 
-    def _get_toprated_category_content(self):
-        toprated_cat = get_category_by_name(self.categories, 
-                                            u"Top Rated")  # unstranslated name
-        if toprated_cat is None:
-            LOG.warn("No 'toprated' category found!!")
-            return None, []
-
-        enq = AppEnquire(self.cache, self.db)
-        app_filter = AppFilter(self.db, self.cache)
-        enq.set_query(toprated_cat.query,
-                      limit=TOP_RATED_CAROUSEL_LIMIT,
-                      sortmode=toprated_cat.sortmode,
-                      filter=app_filter,
-                      nonapps_visible=NonAppVisibility.ALWAYS_VISIBLE,
-                      nonblocking_load=False)
-
-        if not hasattr(self, "helper"):
-            self.helper = AppPropertiesHelper(self.db,
-                                              self.cache,
-                                              self.icons)
-
-        return toprated_cat, enq.get_documents()
-
-    def _update_toprated_content(self):
+    # FIXME: _update_{top_rated,whats_new,recommended_for_you}_content()
+    #        duplicates a lot of code
+    def _update_top_rated_content(self):
         # remove any existing children from the grid widget
-        self.toprated.remove_all()
-        # get toprated category and docs
-        toprated_cat, docs = self._get_toprated_category_content()
-        # display docs
-        self._add_tiles_to_flowgrid(docs, self.toprated,
-                                    TOP_RATED_CAROUSEL_LIMIT)
-        self.toprated.show_all()
-        return toprated_cat
+        self.top_rated.remove_all()
+        # get top_rated category and docs
+        top_rated_cat = get_category_by_name(
+            self.categories, u"Top Rated")  # untranslated name
+        if top_rated_cat:
+            docs = top_rated_cat.get_documents(self.db)
+            self._add_tiles_to_flowgrid(docs, self.top_rated,
+                                        TOP_RATED_CAROUSEL_LIMIT)
+            self.top_rated.show_all()
+        return top_rated_cat
 
     def _append_top_rated(self):
-        self.toprated = FlowableGrid()
-        #~ self.featured.row_spacing = StockEms.SMALL
-        frame = FramedHeaderBox()
-        frame.set_header_label(_("Top Rated"))
-        frame.add(self.toprated)
-        self.toprated_frame = frame
-        self.right_column.pack_start(frame, True, True, 0)
-
-        toprated_cat = self._update_toprated_content()
-        # only display the 'More' LinkButton if we have toprated content
-        if toprated_cat is not None:
-            frame.header_implements_more_button()
-            frame.more.connect('clicked', self.on_category_clicked, toprated_cat) 
+        self.top_rated = FlowableGrid()
+        #~ self.top_rated.row_spacing = StockEms.SMALL
+        self.top_rated_frame = FramedHeaderBox()
+        self.top_rated_frame.set_header_label(_("Top Rated"))
+        self.top_rated_frame.add(self.top_rated)
+        self.right_column.pack_start(self.top_rated_frame, True, True, 0)
+        top_rated_cat = self._update_top_rated_content()
+        # only display the 'More' LinkButton if we have top_rated content
+        if top_rated_cat is not None:
+            self.top_rated_frame.header_implements_more_button()
+            self.top_rated_frame.more.connect('clicked',
+                               self.on_category_clicked, top_rated_cat)
         return
 
-    def _get_new_category_content(self):
-        whatsnew_cat = get_category_by_name(self.categories, 
-                                            u"What\u2019s New") # unstranslated name
-        if whatsnew_cat is None:
-            LOG.warn("No 'new' category found!!")
-            return None, []
-
-        enq = AppEnquire(self.cache, self.db)
-        app_filter = AppFilter(self.db, self.cache)
-        app_filter.set_available_only(True)
-        app_filter.set_not_installed_only(True)
-        enq.set_query(whatsnew_cat.query,
-                      limit=8,
-                      filter=app_filter,
-                      sortmode=SortMethods.BY_CATALOGED_TIME,
-                      nonapps_visible=NonAppVisibility.ALWAYS_VISIBLE,
-                      nonblocking_load=False)
-
-        if not hasattr(self, "helper"):
-            self.helper = AppPropertiesHelper(self.db,
-                                              self.cache,
-                                              self.icons)
-
-        return whatsnew_cat, enq.get_documents()
-
-    def _update_new_content(self):
+    def _update_whats_new_content(self):
         # remove any existing children from the grid widget
-        self.featured.remove_all()
-        # get toprated category and docs
-        whatsnew_cat, docs = self._get_new_category_content()
-        # display docs
-        self._add_tiles_to_flowgrid(docs, self.featured, 8)
-        self.featured.show_all()
-        return whatsnew_cat
+        self.whats_new.remove_all()
+        # get top_rated category and docs
+        whats_new_cat = get_category_by_name(
+            self.categories, u"What\u2019s New")  # untranslated name
+        if whats_new_cat:
+            docs = whats_new_cat.get_documents(self.db)
+            self._add_tiles_to_flowgrid(docs, self.whats_new, 8)
+            self.whats_new.show_all()
+        return whats_new_cat
 
-    def _append_new(self):
-        self.featured = FlowableGrid()
-        frame = FramedHeaderBox()
-        frame.set_header_label(_(u"What\u2019s New"))
-        frame.add(self.featured)
-        self.new_frame = frame
+    def _append_whats_new(self):
+        self.whats_new = FlowableGrid()
+        self.whats_new_frame = FramedHeaderBox()
+        self.whats_new_frame.set_header_label(_(u"What\u2019s New"))
+        self.whats_new_frame.add(self.whats_new)
 
-        whatsnew_cat = self._update_new_content()
-        if whatsnew_cat is not None:
+        whats_new_cat = self._update_whats_new_content()
+        if whats_new_cat is not None:
             # only add to the visible right_frame if we actually have it
-            self.right_column.pack_start(frame, True, True, 0)
-            frame.header_implements_more_button()
-            frame.more.connect('clicked', self.on_category_clicked, whatsnew_cat) 
-        return
+            self.right_column.pack_start(self.whats_new_frame, True, True, 0)
+            self.whats_new_frame.header_implements_more_button()
+            self.whats_new_frame.more.connect(
+                'clicked', self.on_category_clicked, whats_new_cat)
 
-    #~ def _append_recommendations(self):
-        #~ featured_cat = get_category_by_name(self.categories, 
-                                            #~ u"Featured")  # unstranslated name
-#~ 
-        #~ enq = AppEnquire(self.cache, self.db)
-        #~ app_filter = AppFilter(self.db, self.cache)
-        #~ enq.set_query(featured_cat.query,
-                      #~ limit=12,
-                      #~ filter=app_filter,
-                      #~ nonapps_visible=NonAppVisibility.ALWAYS_VISIBLE,
-                      #~ nonblocking_load=False)
-#~ 
-        #~ self.featured = FlowableGrid()
-        #~ frame = FramedHeaderBox(Gtk.Orientation.VERTICAL)
-        #~ frame.add(self.featured)
-        #~ frame.set_header_label(_("Recommended For You"))
-        #~ frame.header_implements_more_button()
-        #~ self.right_column.pack_start(frame, True, True, 0)
-#~ 
-        #~ self.helper = AppPropertiesHelper(self.db, self.cache, self.icons)
-        #~ docs = enq.get_documents()
-        #~ self._add_tiles_to_flowgrid(docs, self.featured, 12)
-        #~ return
+    def _update_recommended_for_you_content(self):
+        if (self.recommended_for_you_panel and
+            self.recommended_for_you_panel.get_parent()):
+            self.bottom_hbox.remove(self.recommended_for_you_panel)
+        self.recommended_for_you_panel = RecommendationsPanelLobby(self)
+        self.bottom_hbox.pack_start(self.recommended_for_you_panel,
+                                    True, True, 0)
+
+    def _append_recommended_for_you(self):
+        # TODO: This space will initially contain an opt-in screen, and this
+        #       will update to the tile view of recommended apps when ready
+        #       see https://wiki.ubuntu.com/SoftwareCenter#Home_screen
+        self.bottom_hbox = Gtk.HBox(spacing=StockEms.SMALL)
+        bottom_hbox_alignment = Gtk.Alignment()
+        bottom_hbox_alignment.set_padding(0, 0, StockEms.MEDIUM - 2,
+            StockEms.MEDIUM - 2)
+        bottom_hbox_alignment.add(self.bottom_hbox)
+        self.vbox.pack_start(bottom_hbox_alignment, False, False, 0)
+
+        # TODO: During development, place the "Recommended For You" panel
+        #       at the bottom, but swap this with the Top Rated panel once
+        #       the recommended for you pieces are done and deployed
+        #       see https://wiki.ubuntu.com/SoftwareCenter#Home_screen
+        self.recommended_for_you_panel = RecommendationsPanelLobby(self)
+        self.bottom_hbox.pack_start(self.recommended_for_you_panel,
+                                    True, True, 0)
 
     def _update_appcount(self):
         enq = AppEnquire(self.cache, self.db)
@@ -497,14 +471,9 @@ class LobbyViewGtk(CategoriesViewGtk):
         else:
             query = xapian.Query('')
 
-        enq.set_query(query,
-                      limit=0,
-                      nonapps_visible=NonAppVisibility.ALWAYS_VISIBLE,
-                      nonblocking_load=True)
-
-        length = len(enq.matches)
+        length = enq.get_estimated_matches_count(query)
         text = gettext.ngettext("%(amount)s item", "%(amount)s items", length
-                                ) % { 'amount' : length, }
+                                ) % {'amount': length}
         self.appcount.set_text(text)
 
     def _append_appcount(self):
@@ -529,12 +498,13 @@ class LobbyViewGtk(CategoriesViewGtk):
             return
         self._supported_only = supported_only
 
-        self._update_toprated_content()
-        self._update_new_content()
+        self._update_top_rated_content()
+        self._update_whats_new_content()
+        self._update_recommended_for_you_content()
         self._update_appcount()
         return
 
-    # stubs for the time being, we may reuse them if we get dynamic content 
+    # stubs for the time being, we may reuse them if we get dynamic content
     # again
     def stop_carousels(self):
         pass
@@ -542,6 +512,7 @@ class LobbyViewGtk(CategoriesViewGtk):
     def start_carousels(self):
         pass
 
+
 class SubCategoryViewGtk(CategoriesViewGtk):
 
     def __init__(self, datadir, desktopdir, cache, db, icons,
@@ -553,23 +524,23 @@ class SubCategoryViewGtk(CategoriesViewGtk):
         # data
         self.root_category = root_category
         self.enquire = AppEnquire(self.cache, self.db)
-        self.helper = AppPropertiesHelper(self.db,
-                                          self.cache,
-                                          self.icons)
+        self.properties_helper = AppPropertiesHelper(
+            self.db, self.cache, self.icons)
 
         # sections
         self.current_category = None
         self.departments = None
-        self.toprated = None
+        self.top_rated = None
+        self.recommended_for_you_in_cat = None
         self.appcount = None
 
         # widgetry
-        self.vbox.set_margin_left(StockEms.MEDIUM-2)
-        self.vbox.set_margin_right(StockEms.MEDIUM-2)
+        self.vbox.set_margin_left(StockEms.MEDIUM - 2)
+        self.vbox.set_margin_right(StockEms.MEDIUM - 2)
         self.vbox.set_margin_top(StockEms.MEDIUM)
         return
 
-    def _get_sub_toprated_content(self, category):
+    def _get_sub_top_rated_content(self, category):
         app_filter = AppFilter(self.db, self.cache)
         self.enquire.set_query(category.query,
                                limit=TOP_RATED_CAROUSEL_LIMIT,
@@ -579,31 +550,51 @@ class SubCategoryViewGtk(CategoriesViewGtk):
                                nonblocking_load=False)
         return self.enquire.get_documents()
 
-    @wait_for_apt_cache_ready # be consistent with new apps
-    def _update_sub_toprated_content(self, category):
-        self.toprated.remove_all()
+    @wait_for_apt_cache_ready  # be consistent with new apps
+    def _update_sub_top_rated_content(self, category):
+        self.top_rated.remove_all()
         # FIXME: should this be m = "%s %s" % (_(gettext text), header text) ??
-        m = _('Top Rated %s') % GObject.markup_escape_text(self.header)
-        self.toprated_frame.set_header_label(m)
-        docs = self._get_sub_toprated_content(category)
-        self._add_tiles_to_flowgrid(docs, self.toprated, TOP_RATED_CAROUSEL_LIMIT)
+        # TRANSLATORS: %s is a category name, like Internet or Development
+        # Tools
+        m = _('Top Rated %(category)s') % {
+            'category': GObject.markup_escape_text(self.header)}
+        self.top_rated_frame.set_header_label(m)
+        docs = self._get_sub_top_rated_content(category)
+        self._add_tiles_to_flowgrid(docs, self.top_rated,
+                                    TOP_RATED_CAROUSEL_LIMIT)
         return
 
-    def _append_sub_toprated(self):
-        self.toprated = FlowableGrid()
-        self.toprated.set_row_spacing(6)
-        self.toprated.set_column_spacing(6)
-        self.toprated_frame = FramedHeaderBox()
-        self.toprated_frame.pack_start(self.toprated, True, True, 0)
-        self.vbox.pack_start(self.toprated_frame, False, True, 0)
+    def _append_sub_top_rated(self):
+        self.top_rated = FlowableGrid()
+        self.top_rated.set_row_spacing(6)
+        self.top_rated.set_column_spacing(6)
+        self.top_rated_frame = FramedHeaderBox()
+        self.top_rated_frame.pack_start(self.top_rated, True, True, 0)
+        self.vbox.pack_start(self.top_rated_frame, False, True, 0)
         return
 
+    def _update_recommended_for_you_in_cat_content(self, category):
+        if (self.recommended_for_you_in_cat and
+            self.recommended_for_you_in_cat.get_parent()):
+            self.vbox.remove(self.recommended_for_you_in_cat)
+        self.recommended_for_you_in_cat = RecommendationsPanelCategory(
+                                                                    self,
+                                                                    category)
+        # only show the panel in the categories view when the user
+        # is opted in to the recommender service
+        # FIXME: this is needed vs. a simple hide() on the widget because
+        #        we do a show_all on the view
+        if self.recommended_for_you_in_cat.recommender_agent.is_opted_in():
+            self.vbox.pack_start(self.recommended_for_you_in_cat,
+                                        False, False, 0)
+
     def _update_subcat_departments(self, category, num_items):
         self.departments.remove_all()
 
         # set the subcat header
         m = "<b><big>%s</big></b>"
-        self.subcat_label.set_markup(m % GObject.markup_escape_text(self.header))
+        self.subcat_label.set_markup(m % GObject.markup_escape_text(
+            self.header))
 
         # sort Category.name's alphabetically
         sorted_cats = categories_sorted_by_name(self.categories)
@@ -613,14 +604,14 @@ class SubCategoryViewGtk(CategoriesViewGtk):
             # add the subcategory if and only if it is non-empty
             enquire.set_query(cat.query)
 
-            if len(enquire.get_mset(0,1)):
+            if len(enquire.get_mset(0, 1)):
                 tile = CategoryTile(cat.name, cat.iconname)
                 tile.connect('clicked', self.on_category_clicked, cat)
                 self.departments.add_child(tile)
 
         # partialy work around a (quite rare) corner case
         if num_items == 0:
-            enquire.set_query(xapian.Query(xapian.Query.OP_AND, 
+            enquire.set_query(xapian.Query(xapian.Query.OP_AND,
                                 category.query,
                                 xapian.Query("ATapplication")))
             # assuming that we only want apps is not always correct ^^^
@@ -628,7 +619,8 @@ class SubCategoryViewGtk(CategoriesViewGtk):
             num_items = tmp_matches.get_matches_estimated()
 
         # append an additional button to show all of the items in the category
-        all_cat = Category("All", _("All"), "category-show-all", category.query)
+        all_cat = Category("All", _("All"), "category-show-all",
+            category.query)
         name = GObject.markup_escape_text('%s %s' % (_("All"), num_items))
         tile = CategoryTile(name, "category-show-all")
         tile.connect('clicked', self.on_category_clicked, all_cat)
@@ -642,20 +634,20 @@ class SubCategoryViewGtk(CategoriesViewGtk):
         self.departments = FlowableGrid(paint_grid_pattern=False)
         self.departments.set_row_spacing(StockEms.SMALL)
         self.departments.set_column_spacing(StockEms.SMALL)
-        frame = FramedBox(spacing=StockEms.MEDIUM,
-                          padding=StockEms.MEDIUM)
+        self.departments_frame = FramedBox(spacing=StockEms.MEDIUM,
+                                           padding=StockEms.MEDIUM)
         # set x/y-alignment and x/y-expand
-        frame.set(0.5, 0.0, 1.0, 1.0)
-        frame.pack_start(self.subcat_label, False, False, 0)
-        frame.pack_start(self.departments, True, True, 0)
+        self.departments_frame.set(0.5, 0.0, 1.0, 1.0)
+        self.departments_frame.pack_start(self.subcat_label, False, False, 0)
+        self.departments_frame.pack_start(self.departments, True, True, 0)
         # append the departments section to the page
-        self.vbox.pack_start(frame, False, True, 0)
+        self.vbox.pack_start(self.departments_frame, False, True, 0)
         return
 
     def _update_appcount(self, appcount):
         text = gettext.ngettext("%(amount)s item available",
                                 "%(amount)s items available",
-                                appcount) % { 'amount' : appcount, }
+                                appcount) % {'amount': appcount}
         self.appcount.set_text(text)
         return
 
@@ -671,14 +663,18 @@ class SubCategoryViewGtk(CategoriesViewGtk):
         # these methods add sections to the page
         # changing order of methods changes order that they appear in the page
         self._append_subcat_departments()
-        self._append_sub_toprated()
+        self._append_sub_top_rated()
+        # NOTE that the recommended for you in category view is built and added
+        # in the _update_recommended_for_you_in_cat method (and so is not
+        # needed here)
         self._append_appcount()
         self._built = True
         return
 
     def _update_subcat_view(self, category, num_items=0):
         num_items = self._update_subcat_departments(category, num_items)
-        self._update_sub_toprated_content(category)
+        self._update_sub_top_rated_content(category)
+        self._update_recommended_for_you_in_cat_content(category)
         self._update_appcount(num_items)
         self.show_all()
         return
@@ -693,7 +689,8 @@ class SubCategoryViewGtk(CategoriesViewGtk):
         self.header = root_category.name
         self.categories = root_category.subcategories
 
-        if not self._built: self._build_subcat_view()
+        if not self._built:
+            self._build_subcat_view()
         self._update_subcat_view(root_category, num_items)
 
         GObject.idle_add(self.queue_draw)
@@ -706,7 +703,8 @@ class SubCategoryViewGtk(CategoriesViewGtk):
             return
         self._supported_only = supported_only
 
-        if not self._built: self._build_subcat_view()
+        if not self._built:
+            self._build_subcat_view()
         self._update_subcat_view(self.current_category)
         GObject.idle_add(self.queue_draw)
         return
@@ -716,8 +714,73 @@ class SubCategoryViewGtk(CategoriesViewGtk):
         #self.set_subcategory(self.root_category)
         #return
 
+
 def get_test_window_catview():
 
+    def on_category_selected(view, cat):
+        print "on_category_selected view: ", view
+        print "on_category_selected cat: ", cat
+
+    from softwarecenter.db.pkginfo import get_pkg_info
+    cache = get_pkg_info()
+    cache.open()
+
+    from softwarecenter.db.database import StoreDatabase
+    xapian_base_path = "/var/cache/software-center"
+    pathname = os.path.join(xapian_base_path, "xapian")
+    db = StoreDatabase(pathname, cache)
+    db.open()
+
+    import softwarecenter.paths
+    datadir = softwarecenter.paths.datadir
+
+    from softwarecenter.ui.gtk3.utils import get_sc_icon_theme
+    icons = get_sc_icon_theme(datadir)
+
+    import softwarecenter.distro
+    distro = softwarecenter.distro.get_distro()
+
+    apps_filter = AppFilter(db, cache)
+
+    # gui
+    win = Gtk.Window()
+    notebook = Gtk.Notebook()
+
+    from softwarecenter.paths import APP_INSTALL_PATH
+    view = LobbyViewGtk(datadir, APP_INSTALL_PATH,
+                        cache, db, icons, distro, apps_filter)
+    win.set_data("lobby", view)
+
+    scroll = Gtk.ScrolledWindow()
+    scroll.add(view)
+    notebook.append_page(scroll, Gtk.Label(label="Lobby"))
+
+    # find a cat in the LobbyView that has subcategories
+    subcat_cat = None
+    for cat in reversed(view.categories):
+        if cat.subcategories:
+            subcat_cat = cat
+            break
+
+    view = SubCategoryViewGtk(datadir, APP_INSTALL_PATH, cache, db, icons,
+                              apps_filter)
+    view.connect("category-selected", on_category_selected)
+    view.set_subcategory(subcat_cat)
+    win.set_data("subcat", view)
+
+    scroll = Gtk.ScrolledWindow()
+    scroll.add(view)
+    notebook.append_page(scroll, Gtk.Label(label="Subcats"))
+
+    win.add(notebook)
+    win.set_size_request(800, 800)
+    win.show_all()
+    win.connect('destroy', Gtk.main_quit)
+    return win
+
+
+def get_test_catview():
+
     def on_category_selected(view, cat):
         print("on_category_selected %s %s" % view, cat)
 
@@ -742,41 +805,10 @@ def get_test_window_catview():
 
     apps_filter = AppFilter(db, cache)
 
-    # gui
-    win = Gtk.Window()
-    n = Gtk.Notebook()
-
     from softwarecenter.paths import APP_INSTALL_PATH
-    view = LobbyViewGtk(datadir, APP_INSTALL_PATH,
+    cat_view = LobbyViewGtk(datadir, APP_INSTALL_PATH,
                         cache, db, icons, distro, apps_filter)
-    win.set_data("lobby", view)
-
-    scroll = Gtk.ScrolledWindow()
-    scroll.add(view)
-    n.append_page(scroll, Gtk.Label(label="Lobby"))
-
-    # find a cat in the LobbyView that has subcategories
-    subcat_cat = None
-    for cat in reversed(view.categories):
-        if cat.subcategories:
-            subcat_cat = cat
-            break
-
-    view = SubCategoryViewGtk(datadir, APP_INSTALL_PATH, cache, db, icons,
-                              apps_filter)
-    view.connect("category-selected", on_category_selected)
-    view.set_subcategory(subcat_cat)
-    win.set_data("subcat", view)
-
-    scroll = Gtk.ScrolledWindow()
-    scroll.add(view)
-    n.append_page(scroll, Gtk.Label(label="Subcats"))
-
-    win.add(n)
-    win.set_size_request(800,600)
-    win.show_all()
-    win.connect('destroy', Gtk.main_quit)
-    return win
+    return cat_view
 
 if __name__ == "__main__":
     import os
@@ -786,5 +818,3 @@ if __name__ == "__main__":
 
     # run it
     Gtk.main()
-
-
