@@ -20,65 +20,32 @@
 from gi.repository import GObject
 from gi.repository import Gtk
 from gi.repository import Gdk
-from gi.repository import WebKit as webkit
+
+import ConfigParser
 import logging
 import os
 import json
 import sys
 import urllib
 
+from gi.repository import WebKit as webkit
+
 from gettext import gettext as _
 
 from softwarecenter.backend import get_install_backend
-from softwarecenter.utils import get_language
+from softwarecenter.ui.gtk3.dialogs import show_accept_tos_dialog
+from softwarecenter.config import get_config
+from softwarecenter.ui.gtk3.utils import get_parent
+from softwarecenter.ui.gtk3.views.webkit import ScrolledWebkitWindow
 
 LOG = logging.getLogger(__name__)
 
-# enable certificates validation in webkit views unless specified otherwise
-if not "SOFTWARE_CENTER_FORCE_DISABLE_CERTS_CHECK" in os.environ:
-    session = webkit.get_default_session()
-    session.set_property("ssl-ca-file", "/etc/ssl/certs/ca-certificates.crt")
-else:
-    # WARN the user!! Do not remove this
-    LOG.warning("SOFTWARE_CENTER_FORCE_DISABLE_CERTS_CHECK " +
-                "has been specified, all purchase transactions " + 
-                "are now INSECURE and UNENCRYPTED!!")
-
-class LocaleAwareWebView(webkit.WebView):
-    
-    def __init__(self):
-        webkit.WebView.__init__(self)
-        self.connect("resource-request-starting", 
-                     self._on_resource_request_starting)
-
-    def _on_resource_request_starting(self, view, frame, res, req, resp):
-        lang = get_language()
-        if lang:
-            message = req.get_message()
-            if message:
-                headers = message.get_property("request-headers")
-                headers.append("Accept-Language", lang)
-        #def _show_header(name, value, data):
-        #    print name, value
-        #headers.foreach(_show_header, None)
-
-class ScrolledWebkitWindow(Gtk.ScrolledWindow):
-
-    def __init__(self):
-        super(ScrolledWebkitWindow, self).__init__()
-        self.webkit = LocaleAwareWebView()
-        settings = self.webkit.get_settings()
-        settings.set_property("enable-plugins", False)
-        self.webkit.show()
-        # embed the webkit view in a scrolled window
-        self.add(self.webkit)
-        self.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
 
 class PurchaseView(Gtk.VBox):
-    """ 
-    View that displays the webkit-based UI for purchasing an item. 
     """
-    
+    View that displays the webkit-based UI for purchasing an item.
+    """
+
     LOADING_HTML = """
 <html>
 <head>
@@ -103,7 +70,8 @@ body {
   vertical-align: middle;
 }
 h1 {
-  background: url(file:///usr/share/software-center/images/spinner.gif) top center no-repeat;
+  background: url(file:///usr/share/software-center/images/spinner.gif) top \
+center no-repeat;
   padding-top: 48px; /* leaves room for the spinner above */
   font-size: 100%%;
   font-weight: normal;
@@ -116,26 +84,36 @@ h1 {
 """ % _("Connecting to payment service...")
 
     __gsignals__ = {
-         'purchase-succeeded' : (GObject.SignalFlags.RUN_LAST,
-                                 None,
-                                 ()),
-         'purchase-failed'    : (GObject.SignalFlags.RUN_LAST,
-                                 None,
-                                 ()),
-         'purchase-cancelled-by-user' : (GObject.SignalFlags.RUN_LAST,
-                                         None,
-                                         ()),
+         'purchase-succeeded': (GObject.SignalFlags.RUN_LAST,
+                                None,
+                                ()),
+         'purchase-failed': (GObject.SignalFlags.RUN_LAST,
+                             None,
+                             ()),
+         'purchase-cancelled-by-user': (GObject.SignalFlags.RUN_LAST,
+                                        None,
+                                        ()),
+         'terms-of-service-declined': (GObject.SignalFlags.RUN_LAST,
+                                       None,
+                                       ()),
+         'purchase-needs-spinner': (GObject.SignalFlags.RUN_LAST,
+                                     None,
+                                    (bool, )),
+
     }
 
     def __init__(self):
         GObject.GObject.__init__(self)
         self.wk = None
         self._wk_handlers_blocked = False
+        self._oauth_token = None
+        self.config = get_config()
 
     def init_view(self):
         if self.wk is None:
             self.wk = ScrolledWebkitWindow()
-            #self.wk.webkit.connect("new-window-policy-decision-requested", self._on_new_window)
+            #self.wk.webkit.connect("new-window-policy-decision-requested",
+            #    self._on_new_window)
             self.wk.webkit.connect("create-web-view", self._on_create_web_view)
             self.wk.webkit.connect("close-web-view", self._on_close_web_view)
             self.wk.webkit.connect("console-message", self._on_console_message)
@@ -143,16 +121,36 @@ h1 {
             # a possible way to do IPC (script or title change)
             self.wk.webkit.connect("script-alert", self._on_script_alert)
             self.wk.webkit.connect("title-changed", self._on_title_changed)
-            self.wk.webkit.connect("notify::load-status", self._on_load_status_changed)
-        # unblock signal handlers if needed when showing the purchase webkit view in
-        # case they were blocked after a previous purchase was completed or canceled
+            self.wk.webkit.connect("notify::load-status",
+                self._on_load_status_changed)
+        # unblock signal handlers if needed when showing the purchase webkit
+        # view in case they were blocked after a previous purchase was
+        # completed or canceled
         self._unblock_wk_handlers()
 
+    def _ask_for_tos_acceptance_if_needed(self):
+        try:
+            accepted_tos = self.config.getboolean("general", "accepted_tos")
+        except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
+            accepted_tos = False
+        if not accepted_tos:
+            # show the dialog and ensure the user accepts it
+            res = show_accept_tos_dialog(get_parent(self))
+            if not res:
+                return False
+            self.config.set("general", "accepted_tos", "yes")
+            return True
+        return True
+
     def initiate_purchase(self, app, iconname, url=None, html=None):
         """
         initiates the purchase workflow inside the embedded webkit window
-        for the item specified       
+        for the item specified
         """
+        if not self._ask_for_tos_acceptance_if_needed():
+            self.emit("terms-of-service-declined")
+            return False
+
         self.init_view()
         self.app = app
         self.iconname = iconname
@@ -170,7 +168,8 @@ h1 {
         # only for debugging
         if os.environ.get("SOFTWARE_CENTER_DEBUG_BUY"):
             GObject.timeout_add_seconds(1, _generate_events, self)
-        
+        return True
+
     def _on_new_window(self, view, frame, request, action, policy):
         LOG.debug("_on_new_window")
         import subprocess
@@ -181,28 +180,35 @@ h1 {
         win = view.get_data("win")
         win.destroy()
         return True
-        
+
     def _on_create_web_view(self, view, frame):
         win = Gtk.Window()
-        win.set_size_request(500, 400)
-        # set transient parent
-        parent = self
-        while parent.get_parent():
-            parent = parent.get_parent()
-        win.set_transient_for(parent)
-        # and add the webkit view
-        wk = ScrolledWebkitWindow()
+        win.set_size_request(400, 400)
+        wk = ScrolledWebkitWindow(include_progress_ui=True)
         wk.webkit.connect("close-web-view", self._on_close_web_view)
         win.add(wk)
         win.show_all()
         # make sure close will work later
         wk.webkit.set_data("win", win)
+        # find and set parent
+        w = self.wk.get_parent()
+        while w.get_parent():
+            w = w.get_parent()
+        win.set_transient_for(w)
         return wk.webkit
 
     def _on_console_message(self, view, message, line, source_id):
+        try:
+            # load the token from the console message
+            self._oauth_token = json.loads(message)
+            # compat with the regular oauth naming
+            self._oauth_token["token"] = self._oauth_token["token_key"]
+        except ValueError:
+            pass
         for k in ["token_key", "token_secret", "consumer_secret"]:
             if k in message:
-                LOG.debug("skipping console message that contains sensitive data")
+                LOG.debug(
+                    "skipping console message that contains sensitive data")
                 return True
         LOG.debug("_on_console_message '%s'" % message)
         return False
@@ -222,10 +228,13 @@ h1 {
         prop = view.get_property(property_spec.name)
         window = self.get_window()
         if prop == webkit.LoadStatus.PROVISIONAL:
+            self.emit("purchase-needs-spinner", True)
             if window:
                 window.set_cursor(Gdk.Cursor.new(Gdk.CursorType.WATCH))
         elif (prop == webkit.LoadStatus.FIRST_VISUALLY_NON_EMPTY_LAYOUT or
+              prop == webkit.LoadStatus.FAILED or
               prop == webkit.LoadStatus.FINISHED):
+            self.emit("purchase-needs-spinner", False)
             if window:
                 window.set_cursor(None)
 
@@ -250,7 +259,8 @@ h1 {
                 return
             # this is what the agent implements
             elif "failures" in res:
-                LOG.error("the server returned a error: '%s'" % res["failures"])
+                LOG.error("the server returned a error: '%s'" %
+                    res["failures"])
             # show a generic error, the "failures" string we get from the
             # server is way too technical to show, but we do log it
             self.emit("purchase-failed")
@@ -267,9 +277,9 @@ h1 {
             # add repo and key
             backend = get_install_backend()
             backend.add_repo_add_key_and_install_app(
-                deb_line, signing_key_id, self.app, self.iconname, 
-                license_key, license_key_path)
-                                                                   
+                deb_line, signing_key_id, self.app, self.iconname,
+                license_key, license_key_path, json.dumps(self._oauth_token))
+
     def _block_wk_handlers(self):
         # we need to block webkit signal handlers when we hide the
         # purchase webkit view, this prevents e.g. handling of signals on
@@ -279,14 +289,15 @@ h1 {
             self.wk.webkit.handler_block_by_func(self._on_title_changed)
             self.wk.webkit.handler_block_by_func(self._on_load_status_changed)
             self._wk_handlers_blocked = True
-        
+
     def _unblock_wk_handlers(self):
         if self._wk_handlers_blocked:
             self.wk.webkit.handler_unblock_by_func(self._on_script_alert)
             self.wk.webkit.handler_unblock_by_func(self._on_title_changed)
-            self.wk.webkit.handler_unblock_by_func(self._on_load_status_changed)
+            self.wk.webkit.handler_unblock_by_func(
+                self._on_load_status_changed)
             self._wk_handlers_blocked = False
-        
+
 
 # just used for testing --------------------------------------------
 DUMMY_HTML = """
@@ -299,11 +310,11 @@ DUMMY_HTML = """
 <body>
  <script type="text/javascript">
   function changeTitle(title) { document.title = title; }
-  function success() { changeTitle('{ "successful" : true, \
-                                      "deb_line" : "deb https://user:pass@private-ppa.launchpad.net/mvo/ubuntu lucid main", \
-                                      "package_name" : "2vcard", \
-                                      "application_name" : "The 2vcard app", \
-                                      "signing_key_id" : "1024R/0EB12F05"\
+  function success() { changeTitle('{"successful": true, "deb_line": \
+    "deb https://user:pass@private-ppa.launchpad.net/mvo/ubuntu lucid main", \
+   "package_name": "2vcard", \
+   "application_name": "The 2vcard app", \
+   "signing_key_id": "1024R/0EB12F05"\
                                     }') }
   function cancel() { changeTitle('{ "successful" : false }') }
  </script>
@@ -313,11 +324,11 @@ DUMMY_HTML = """
  <p>
  <input type="entry">
  </p>
- <input type="button" name="test_button2" 
+ <input type="button" name="test_button2"
         value="Cancel"
       onclick='cancel()'
  />
- <input type="button" name="test_button" 
+ <input type="button" name="test_button"
         value="Buy now"
       onclick='success()'
  />
@@ -325,30 +336,31 @@ DUMMY_HTML = """
 </html>
     """
 
+
 # synthetic key event generation
 def _send_keys(view, s):
     print("_send_keys %s" % s)
-    MAPPING = { '@'     : 'at',
-                '.'     : 'period',
-                '\t'    : 'Tab',
-                '\n'    : 'Return',
-                '?'     : 'question',
-                '\a'    : 'Down',  # fake 
-                ' '     : 'space',
-                '\v'    : 'Page_Down', # fake
+    MAPPING = {'@': 'at',
+               '.': 'period',
+               '\t': 'Tab',
+               '\n': 'Return',
+               '?': 'question',
+               '\a': 'Down',  # fake
+               ' ': 'space',
+               '\v': 'Page_Down',  # fake
               }
-    
+
     for key in s:
         event = Gdk.Event(Gdk.KEY_PRESS)
         event.window = view.window
         if key.isdigit():
-            key = "_"+key
+            key = "_" + key
         if hasattr(Gdk, key):
             event.keyval = getattr(Gdk, key)
         else:
             event.keyval = getattr(Gdk, MAPPING[key])
         Gtk.main_do_event(event)
-  
+
 
 # \a means down key - its a just a fake to get it working
 LOGIN = os.environ.get("SOFTWARE_CENTER_LOGIN") or "michael.vogt@ubuntu.com"
@@ -357,12 +369,14 @@ LOGIN = os.environ.get("SOFTWARE_CENTER_LOGIN") or "michael.vogt@ubuntu.com"
 PAYMENT_DETAILS = "\tstreet1\tstreet2\tcity\tstate\t1234\t\a\t\a\a\t"\
                   "ACCEPTED\t4111111111111111\t1234\t\a\t\a\a\t\t\t \v"
 # state-name, window title, keys
-STATES = [ ('login', 'Log in', LOGIN+"\t"),
-           ('confirm-sso', 'Authenticate to', '\n'),
-           ('enter-payment', 'Confirm Payment Details', PAYMENT_DETAILS),
-           ('confirm-payment', 'title-the-same-as-before', '\t\n'),
-           ('end-state', 'no-title', ''),
+STATES = [('login', 'Log in', LOGIN + "\t"),
+          ('confirm-sso', 'Authenticate to', '\n'),
+          ('enter-payment', 'Confirm Payment Details', PAYMENT_DETAILS),
+          ('confirm-payment', 'title-the-same-as-before', '\t\n'),
+          ('end-state', 'no-title', ''),
          ]
+
+
 def _generate_events(view):
     global STATES
 
@@ -378,19 +392,20 @@ def _generate_events(view):
 
     return True
 
-#     # for debugging only    
+#     # for debugging only
 #    def _on_key_press(dialog, event):
 #        print event, event.keyval
 
+
 def get_test_window_purchaseview():
     #url = "http://www.animiertegifs.de/java-scripts/alertbox.php"
     url = "http://www.ubuntu.cohtml=DUMMY_m"
     #d = PurchaseDialog(app=None, url="http://spiegel.de")
     from softwarecenter.enums import BUY_SOMETHING_HOST
-    url = BUY_SOMETHING_HOST+"/subscriptions/en/ubuntu/maverick/+new/?%s" % ( 
+    url = BUY_SOMETHING_HOST + "/subscriptions/en/ubuntu/maverick/+new/?%s" % (
         urllib.urlencode({
-                'archive_id' : "mvo/private-test", 
-                'arch' : "i386",
+                'archive_id': "mvo/private-test",
+                'arch': "i386",
                 }))
     # use cmdline if available
     if len(sys.argv) > 1:
@@ -398,10 +413,10 @@ def get_test_window_purchaseview():
     # useful for debugging
     #d.connect("key-press-event", _on_key_press)
     #GObject.timeout_add_seconds(1, _generate_events, d)
-    
+
     widget = PurchaseView()
-    widget.initiate_purchase(app=None, iconname=None, url=url)
-    #widget.initiate_purchase(app=None, iconname=None, html=DUMMY_HTML)
+    from mock import Mock
+    widget.config = Mock()
 
     win = Gtk.Window()
     win.set_data("view", widget)
@@ -410,9 +425,14 @@ def get_test_window_purchaseview():
     win.set_position(Gtk.WindowPosition.CENTER)
     win.show_all()
     win.connect('destroy', Gtk.main_quit)
+
+    widget.initiate_purchase(app=None, iconname=None, url=url)
+    #widget.initiate_purchase(app=None, iconname=None, html=DUMMY_HTML)
+
     return win
 
 if __name__ == "__main__":
+    import softwarecenter.paths
+    softwarecenter.paths.datadir = "./data"
     win = get_test_window_purchaseview()
     Gtk.main()
-
