@@ -46,7 +46,7 @@ def _update_icon(image, icon, icon_size):
 
 class _Tile(object):
 
-    MIN_WIDTH  = em(7)
+    MIN_WIDTH = em(7)
 
     def __init__(self):
         self.set_focus_on_click(False)
@@ -54,7 +54,6 @@ class _Tile(object):
         self.box = Gtk.Box.new(Gtk.Orientation.VERTICAL, 0)
         self.box.set_size_request(self.MIN_WIDTH, -1)
         self.add(self.box)
-        return
 
     def build_default(self, label, icon, icon_size):
         if icon is not None:
@@ -67,7 +66,6 @@ class _Tile(object):
 
         self.label = Gtk.Label.new(label)
         self.box.pack_start(self.label, True, True, 0)
-        return
 
 
 class TileButton(Gtk.Button, _Tile):
@@ -75,7 +73,6 @@ class TileButton(Gtk.Button, _Tile):
     def __init__(self):
         Gtk.Button.__init__(self)
         _Tile.__init__(self)
-        return
 
 
 class TileToggleButton(Gtk.RadioButton, _Tile):
@@ -84,7 +81,6 @@ class TileToggleButton(Gtk.RadioButton, _Tile):
         Gtk.RadioButton.__init__(self)
         self.set_mode(False)
         _Tile.__init__(self)
-        return
 
 
 class LabelTile(TileButton):
@@ -101,7 +97,6 @@ class LabelTile(TileButton):
 
         self.connect("enter-notify-event", self.on_enter)
         self.connect("leave-notify-event", self.on_leave)
-        return
 
     def do_draw(self, cr):
         cr.save()
@@ -111,22 +106,20 @@ class LabelTile(TileButton):
             Gtk.render_focus(self.get_style_context(),
                              cr,
                              3, 3,
-                             A.width-6, A.height-6)
+                             A.width - 6, A.height - 6)
 
-        for child in self: self.propagate_draw(child, cr)
+        for child in self:
+            self.propagate_draw(child, cr)
 
         cr.restore()
-        return
 
     def on_enter(self, widget, event):
         window = self.get_window()
         window.set_cursor(_HAND)
-        return
 
     def on_leave(self, widget, event):
         window = self.get_window()
         window.set_cursor(None)
-        return
 
 
 class CategoryTile(TileButton):
@@ -145,7 +138,6 @@ class CategoryTile(TileButton):
 
         self.connect("enter-notify-event", self.on_enter)
         self.connect("leave-notify-event", self.on_leave)
-        return
 
     def do_draw(self, cr):
         cr.save()
@@ -155,25 +147,25 @@ class CategoryTile(TileButton):
             Gtk.render_focus(self.get_style_context(),
                              cr,
                              3, 3,
-                             A.width-6, A.height-6)
+                             A.width - 6, A.height - 6)
 
-        for child in self: self.propagate_draw(child, cr)
+        for child in self:
+            self.propagate_draw(child, cr)
 
         cr.restore()
-        return
 
     def on_enter(self, widget, event):
         window = self.get_window()
         window.set_cursor(_HAND)
-        return
 
     def on_leave(self, widget, event):
         window = self.get_window()
         window.set_cursor(None)
-        return
 
 
 _global_featured_tile_width = em(11)
+
+
 class FeaturedTile(TileButton):
 
     INSTALLED_OVERLAY_SIZE = 22
@@ -186,17 +178,18 @@ class FeaturedTile(TileButton):
         label = helper.get_appname(doc)
         icon = helper.get_icon_at_size(doc, icon_size, icon_size)
         stats = helper.get_review_stats(doc)
-        doc.installed = doc.available = None
+        helper.update_availability(doc)
         helper.connect("needs-refresh", self._on_needs_refresh, doc, icon_size)
         self.is_installed = helper.is_installed(doc)
         self._overlay = helper.icons.load_icon(Icons.INSTALLED_OVERLAY,
                                                self.INSTALLED_OVERLAY_SIZE,
-                                               0) # flags
+                                               0)  # flags
 
         self.box.set_orientation(Gtk.Orientation.HORIZONTAL)
         self.box.set_spacing(StockEms.SMALL)
 
-        self.content_left = Gtk.Box.new(Gtk.Orientation.VERTICAL, StockEms.MEDIUM)
+        self.content_left = Gtk.Box.new(Gtk.Orientation.VERTICAL,
+            StockEms.MEDIUM)
         self.content_right = Gtk.Box.new(Gtk.Orientation.VERTICAL, 1)
         self.box.pack_start(self.content_left, False, False, 0)
         self.box.pack_start(self.content_right, False, False, 0)
@@ -204,7 +197,8 @@ class FeaturedTile(TileButton):
         _update_icon(self.image, icon, icon_size)
         self.content_left.pack_start(self.image, False, False, 0)
 
-        self.title = Gtk.Label.new(self._MARKUP % GObject.markup_escape_text(label))
+        self.title = Gtk.Label.new(self._MARKUP %
+            GObject.markup_escape_text(label))
         self.title.set_alignment(0.0, 0.5)
         self.title.set_use_markup(True)
         self.title.set_ellipsize(Pango.EllipsizeMode.END)
@@ -212,7 +206,8 @@ class FeaturedTile(TileButton):
 
         categories = helper.get_categories(doc)
         if categories is not None:
-            self.category = Gtk.Label.new('<span font_desc="%i">%s</span>' % (em(0.6), GObject.markup_escape_text(categories)))
+            self.category = Gtk.Label.new('<span font_desc="%i">%s</span>' %
+                (em(0.6), GObject.markup_escape_text(categories)))
             self.category.set_use_markup(True)
             self.category.set_alignment(0.0, 0.5)
             self.category.set_ellipsize(Pango.EllipsizeMode.END)
@@ -223,10 +218,11 @@ class FeaturedTile(TileButton):
             self.stars = Star(size=StarSize.SMALL)
             self.stars.render_outline = True
             self.stars.set_rating(stats.ratings_average)
-            self.rating_box = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, StockEms.SMALL)
+            self.rating_box = Gtk.Box.new(Gtk.Orientation.HORIZONTAL,
+                StockEms.SMALL)
             self.rating_box.pack_start(self.stars, False, False, 0)
             self.n_ratings = Gtk.Label.new(
-                '<span font_desc="%i"> (%i)</span>' %  (
+                '<span font_desc="%i"> (%i)</span>' % (
                     em(0.45), stats.ratings_total))
             self.n_ratings.set_use_markup(True)
             self.n_ratings.set_name("subtle-label")
@@ -237,8 +233,9 @@ class FeaturedTile(TileButton):
             # is not visible in the ui
             stats_a11y = _('%(stars)d stars - %(reviews)d reviews') % {
                 'stars': stats.ratings_average, 'reviews': stats.ratings_total}
-        
-            #work out width tile needs to be to ensure ratings text is all visible
+
+            # work out width tile needs to be to ensure ratings text is all
+            # visible
             req_width = (self.stars.size_request().width +
                          self.image.size_request().width +
                          self.n_ratings.size_request().width +
@@ -249,9 +246,12 @@ class FeaturedTile(TileButton):
                                               req_width)
 
         details = AppDetails(db=helper.db, doc=doc)
+        # TRANSLATORS: Free here means Gratis
         price = details.price or _("Free")
         if price == '0.00':
+            # TRANSLATORS: Free here means Gratis
             price = _("Free")
+        # TRANSLATORS: Free here means Gratis
         if price != _("Free"):
             price = 'US$ ' + price
         self.price = Gtk.Label.new(
@@ -263,7 +263,8 @@ class FeaturedTile(TileButton):
 
         self.set_name("featured-tile")
 
-        a11y_name = '. '.join([t for t in [label, categories, stats_a11y, price] if t])    
+        a11y_name = '. '.join([t
+            for t in [label, categories, stats_a11y, price] if t])
         self.get_accessible().set_name(a11y_name)
 
         backend = get_install_backend()
@@ -275,12 +276,10 @@ class FeaturedTile(TileButton):
         self.connect("leave-notify-event", self.on_leave)
         self.connect("button-press-event", self.on_press)
         self.connect("button-release-event", self.on_release)
-        return
 
     def _on_needs_refresh(self, helper, pkgname, doc, icon_size):
         icon = helper.get_icon_at_size(doc, icon_size, icon_size)
         _update_icon(self.image, icon, icon_size)
-        
 
     def do_get_preferred_width(self):
         w = _global_featured_tile_width
@@ -296,9 +295,10 @@ class FeaturedTile(TileButton):
             Gtk.render_focus(self.get_style_context(),
                              cr,
                              3, 3,
-                             A.width-6, A.height-6)
+                             A.width - 6, A.height - 6)
 
-        for child in self: self.propagate_draw(child, cr)
+        for child in self:
+            self.propagate_draw(child, cr)
 
         if self.is_installed:
             # paint installed tick overlay
@@ -312,18 +312,17 @@ class FeaturedTile(TileButton):
             cr.paint()
 
         cr.restore()
-        return
 
     def on_transaction_finished(self, backend, result, helper, doc):
         trans_pkgname = str(result.pkgname)
         pkgname = helper.get_pkgname(doc)
-        if trans_pkgname != pkgname: return
+        if trans_pkgname != pkgname:
+            return
 
         # update installed state
         helper.update_availability(doc)
         self.is_installed = helper.is_installed(doc)
         self.queue_draw()
-        return
 
     def on_enter(self, widget, event):
         window = self.get_window()
@@ -338,13 +337,12 @@ class FeaturedTile(TileButton):
 
     def on_press(self, widget, event):
         self._pressed = True
-        return
 
     def on_release(self, widget, event):
-        if not self._pressed: return
+        if not self._pressed:
+            return
         self.emit("clicked")
         self._pressed = False
-        return
 
 
 class ChannelSelector(Gtk.Button):
@@ -360,23 +358,24 @@ class ChannelSelector(Gtk.Button):
         self.arrow = Gtk.Arrow.new(Gtk.ArrowType.DOWN, Gtk.ShadowType.IN)
         alignment.add(self.arrow)
 
+        # vars
+        self.parent_style_type = Gtk.Toolbar
         self.section_button = section_button
         self.popup = None
         self.connect("button-press-event", self.on_button_press)
-        return
 
     def do_draw(self, cr):
         cr.save()
 
-        toolbar = self.get_ancestor('GtkToolbar')
-        context = toolbar.get_style_context()
+        parent_style = self.get_ancestor(self.parent_style_type)
+        context = parent_style.get_style_context()
 
         color = darken(context.get_border_color(Gtk.StateFlags.ACTIVE), 0.2)
 
         cr.set_line_width(1)
 
         a = self.get_allocation()
-        lin = cairo.LinearGradient(0,0,0,a.height)
+        lin = cairo.LinearGradient(0, 0, 0, a.height)
         lin.add_color_stop_rgba(0.1,
                                 color.red,
                                 color.green,
@@ -404,24 +403,22 @@ class ChannelSelector(Gtk.Button):
 
         cr.restore()
 
-        for child in self: self.propagate_draw(child, cr)
-        return
+        for child in self:
+            self.propagate_draw(child, cr)
 
     def on_button_press(self, button, event):
         if self.popup is None:
             self.build_channel_selector()
         self.show_channel_sel_popup(self, event)
-        return
-#~ 
+#~
     #~ def on_style_updated(self, widget):
         #~ context = widget.get_style_context()
         #~ context.save()
         #~ context.add_class("menu")
         #~ bgcolor = context.get_background_color(Gtk.StateFlags.NORMAL)
         #~ context.restore()
-#~ 
+#~
         #~ self._dark_color = darken(bgcolor, 0.5)
-        #~ return
 
     def show_channel_sel_popup(self, widget, event):
 
@@ -431,29 +428,27 @@ class ChannelSelector(Gtk.Button):
             else:
                 tmpx = a.x + a.width - self.popup.get_allocation().width
             x, y = window.get_root_coords(tmpx,
-                                          a.y + a.height + 1)
+                                          a.y + a.height)
             return (x, y, False)
 
         a = self.section_button.get_allocation()
         window = self.section_button.get_window()
         self.popup.popup(None, None, position_func, (window, a),
                          event.button, event.time)
-        return
 
     def set_build_func(self, build_func):
         self.build_func = build_func
-        return
 
     def build_channel_selector(self):
         self.popup = Gtk.Menu()
+        self.popup.set_name('toolbar-popup')  # to set 'padding: 0;'
+        self.popup.get_style_context().add_class('primary-toolbar')
         self.build_func(self.popup)
-        self.popup.attach_to_widget(self, None)
-        return
 
 
 class SectionSelector(TileToggleButton):
 
-    MIN_WIDTH  = em(5)
+    MIN_WIDTH = em(5)
     _MARKUP = '<small>%s</small>'
 
     def __init__(self, label, icon, icon_size=Gtk.IconSize.DIALOG):
@@ -465,7 +460,7 @@ class SectionSelector(TileToggleButton):
 
         context = self.get_style_context()
         context.add_class("section-sel-bg")
-        
+
         context = self.label.get_style_context()
         context.add_class("section-sel")
 
@@ -475,23 +470,20 @@ class SectionSelector(TileToggleButton):
 
         self.connect('size-allocate', self.on_size_allocate)
         self.connect('style-updated', self.on_style_updated)
-        return
 
     def on_size_allocate(self, *args):
         alloc = self.get_allocation()
-        
+
         if (self._alloc is None or
             self._alloc.width != alloc.width or
             self._alloc.height != alloc.height):
             self._alloc = alloc
             # reset the bg cache
             self._bg_cache = {}
-        return
 
     def on_style_updated(self, *args):
         # also reset the bg cache
         self._bg_cache = {}
-        return
 
     def _cache_bg_for_state(self, state):
         a = self.get_allocation()
@@ -506,9 +498,9 @@ class SectionSelector(TileToggleButton):
         context.set_state(state)
 
         Gtk.render_background(context, cr,
-                          -5, -5, a.width+10, a.height+10)
+                          -5, -5, a.width + 10, a.height + 10)
         Gtk.render_frame(context, cr,
-                          -5, -5, a.width+10, a.height+10)
+                          -5, -5, a.width + 10, a.height + 10)
         del cr
 
         # new surface which will be cached which
@@ -518,11 +510,11 @@ class SectionSelector(TileToggleButton):
 
         # gradient for masking
         lin = cairo.LinearGradient(0, 0, 0, a.height)
-        lin.add_color_stop_rgba(0.0, 1,1,1, 0.1)
-        lin.add_color_stop_rgba(0.25, 1,1,1, 0.7)
-        lin.add_color_stop_rgba(0.5, 1,1,1, 1.0)
-        lin.add_color_stop_rgba(0.75, 1,1,1, 0.7)
-        lin.add_color_stop_rgba(1.0, 1,1,1, 0.1)
+        lin.add_color_stop_rgba(0.0, 1, 1, 1, 0.1)
+        lin.add_color_stop_rgba(0.25, 1, 1, 1, 0.7)
+        lin.add_color_stop_rgba(0.5, 1, 1, 1, 1.0)
+        lin.add_color_stop_rgba(0.75, 1, 1, 1, 0.7)
+        lin.add_color_stop_rgba(1.0, 1, 1, 1, 0.1)
 
         cr.set_source_surface(_surf, 0, 0)
         cr.mask(lin)
@@ -530,7 +522,6 @@ class SectionSelector(TileToggleButton):
 
         # cache the resulting surf...
         self._bg_cache[state] = surf
-        return
 
     def do_draw(self, cr):
         state = self.get_state_flags()
@@ -541,36 +532,47 @@ class SectionSelector(TileToggleButton):
             cr.set_source_surface(self._bg_cache[state], 0, 0)
             cr.paint()
 
-        for child in self: self.propagate_draw(child, cr)
-        return
+        for child in self:
+            self.propagate_draw(child, cr)
 
 
 class Link(Gtk.Label):
 
     __gsignals__ = {
-        "clicked" : (GObject.SignalFlags.RUN_LAST,
-                     None, 
-                     (),)
+        "clicked": (GObject.SignalFlags.RUN_LAST,
+                    None,
+                    (),)
         }
 
     def __init__(self, markup="", uri="none"):
         Gtk.Label.__init__(self)
         self._handler = 0
         self.set_markup(markup, uri)
-        return
 
     def set_markup(self, markup="", uri="none"):
-        if self._handler > 0:
-            GObject.source_remove(self._handler)
-            self._handler = 0
+        markup = '<a href="%s">%s</a>' % (uri, markup)
+        Gtk.Label.set_markup(self, markup)
+        if self._handler == 0:
+            self._handler = self.connect("activate-link",
+                self.on_activate_link)
 
-        Gtk.Label.set_markup(self, '<a href="%s">%s</a>' % (uri, markup))
-        self._handler = self.connect("activate-link", self.on_activate_link)
-        return
+    # synonyms for set_markup
+    def set_label(self, label):
+        return self.set_markup(label)
+
+    def set_text(self, text):
+        return self.set_markup(text)
 
     def on_activate_link(self, uri, data):
         self.emit("clicked")
-        return
+
+    def disable(self):
+        self.set_sensitive(False)
+        self.set_name("subtle-label")
+
+    def enable(self):
+        self.set_sensitive(True)
+        self.set_name("label")
 
 
 class MoreLink(Gtk.Button):
@@ -587,7 +589,6 @@ class MoreLink(Gtk.Button):
         self._init_event_handling()
         context = self.get_style_context()
         context.add_class("more-link")
-        return
 
     def _init_event_handling(self):
         self.connect("enter-notify-event", self.on_enter)
@@ -601,32 +602,51 @@ class MoreLink(Gtk.Button):
             e = layout.get_pixel_extents()[1]
             xo, yo = self.label.get_layout_offsets()
             Gtk.render_focus(self.get_style_context(), cr,
-                             xo-a.x-3, yo-a.y-1,
-                             e.width+6, e.height+2)
+                             xo - a.x - 3, yo - a.y - 1,
+                             e.width + 6, e.height + 2)
 
-        for child in self: self.propagate_draw(child, cr)
-        return
+        for child in self:
+            self.propagate_draw(child, cr)
 
     def on_enter(self, widget, event):
         window = self.get_window()
         window.set_cursor(_HAND)
-        return
 
     def on_leave(self, widget, event):
         window = self.get_window()
         window.set_cursor(None)
-        return
+
+
+def _build_channels_list(popup):
+    for i in range(3):
+        item = Gtk.MenuItem.new()
+        label = Gtk.Label.new("channel_name %s" % i)
+        box = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, StockEms.MEDIUM)
+        box.pack_start(label, False, False, 0)
+        item.add(box)
+        item.show_all()
+        popup.attach(item, 0, 1, i, i + 1)
 
 
 def get_test_buttons_window():
     win = Gtk.Window()
-    win.set_size_request(200,200)
+    win.set_size_request(200, 200)
 
     vb = Gtk.VBox(spacing=12)
     win.add(vb)
 
     link = Link("<small>test link</small>", uri="www.google.co.nz")
-    vb.add(link)
+    vb.pack_start(link, False, False, 0)
+
+    button = Gtk.Button()
+    button.set_label("channels")
+    channels_button = ChannelSelector(button)
+    channels_button.parent_style_type = Gtk.Window
+    channels_button.set_build_func(_build_channels_list)
+    hb = Gtk.HBox()
+    hb.pack_start(button, False, False, 0)
+    hb.pack_start(channels_button, False, False, 0)
+    vb.pack_start(hb, False, False, 0)
 
     win.show_all()
     win.connect("destroy", Gtk.main_quit)
