@@ -68,7 +68,6 @@ class EventHelper(dict):
     def __init__(self):
         dict.__init__(self)
         self.new_press(None, None, None, False)
-        return
 
     def __setitem__(self, k, v):
         if k not in EventHelper.VALID_KEYS:
@@ -80,7 +79,7 @@ class EventHelper(dict):
         if event is None:
             self['event'] = None
         else:
-            # this should be simply event.copy() but that appears broken 
+            # this should be simply event.copy() but that appears broken
             # currently(?)
             self['event'] = EventHelper.ButtonEvent(event)
 
@@ -89,7 +88,6 @@ class EventHelper(dict):
         self['within-selection'] = within_sel
         self['drag-active'] = False
         self['drag-context'] = None
-        return
 
 
 class PangoLayoutProxy(object):
@@ -99,7 +97,6 @@ class PangoLayoutProxy(object):
 
     def __init__(self, context):
         self._layout = Pango.Layout.new(context)
-        return
 
     def xy_to_index(self, x, y):
         return self._layout.xy_to_index(x, y)
@@ -110,7 +107,7 @@ class PangoLayoutProxy(object):
     # setter proxies
     def set_attributes(self, attrs):
         return self._layout.set_attributes(attrs)
-    
+
     def set_markup(self, markup):
         return self._layout.set_markup(markup, -1)
 
@@ -154,7 +151,6 @@ class Layout(PangoLayoutProxy):
         self.allocation = Gdk.Rectangle()
         self._default_attrs = True
         self.set_markup(text)
-        return
 
     def __len__(self):
         return self.length
@@ -169,7 +165,6 @@ class Layout(PangoLayoutProxy):
         a.y = y
         a.width = w
         a.height = h
-        return
 
     def get_position(self):
         return self.allocation.x, self.allocation.y
@@ -182,7 +177,7 @@ class Layout(PangoLayoutProxy):
         if target_x >= 0:
             x = target_x
 
-        y -= _PS*self.widget.line_height
+        y -= _PS * self.widget.line_height
         return layout.xy_to_index(x, y), (x, y)
 
     def cursor_down(self, cursor, target_x=-1):
@@ -193,36 +188,37 @@ class Layout(PangoLayoutProxy):
         if target_x >= 0:
             x = target_x
 
-        y += _PS*self.widget.line_height
+        y += _PS * self.widget.line_height
         return layout.xy_to_index(x, y), (x, y)
 
     def index_at(self, px, py):
         #wa = self.widget.get_allocation()
-        x, y = self.get_position() # layout allocation
-        (_, index, k) = self.xy_to_index((px-x)*_PS, (py-y)*_PS)
+        x, y = self.get_position()  # layout allocation
+        (_, index, k) = self.xy_to_index((px - x) * _PS, (py - y) * _PS)
         return point_in(self.allocation, px, py), index + k
 
     def reset_attrs(self):
         #~ self.set_attributes(Pango.AttrList())
         self.set_markup(self.get_text())
         self._default_attrs = True
-        return
 
     def highlight(self, start, end, bg, fg):
         # FIXME: AttrBackground doesnt seem to be expose by gi yet??
         #~ attrs = Pango.AttrList()
-        #~ attrs.insert(Pango.AttrBackground(bg.red, bg.green, bg.blue, start, end))
-        #~ attrs.insert(Pango.AttrForeground(fg.red, fg.green, fg.blue, start, end))
+        #~ attrs.insert(Pango.AttrBackground(bg.red, bg.green, bg.blue, start,
+        #~     end))
+        #~ attrs.insert(Pango.AttrForeground(fg.red, fg.green, fg.blue, start,
+        #~     end))
         #~ self.set_attributes(attrs)
 
         # XXX: workaround
         text = self.get_text()
-        new_text = text[:start] + '<span background="%s" foreground="%s">' % (bg, fg)
+        new_text = (text[:start] + '<span background="%s" foreground="%s">' %
+            (bg, fg))
         new_text += text[start:end]
         new_text += '</span>' + text[end:]
         self.set_markup(new_text)
         self._default_attrs = False
-        return
 
     def highlight_all(self, bg, fg):
         # FIXME: AttrBackground doesnt seem to be expose by gi yet??
@@ -233,9 +229,9 @@ class Layout(PangoLayoutProxy):
 
         # XXX: workaround
         text = self.get_text()
-        self.set_markup('<span background="%s" foreground="%s">%s</span>' % (bg, fg, text))
+        self.set_markup('<span background="%s" foreground="%s">%s</span>' %
+            (bg, fg, text))
         self._default_attrs = False
-        return
 
 
 class Cursor(object):
@@ -258,7 +254,6 @@ class Cursor(object):
         other_pos = cursor.get_position()
         self.set_position(*other_pos)
         cursor.set_position(*this_pos)
-        return
 
     def same_line(self, cursor):
         return self.get_current_line()[0] == cursor.get_current_line()[0]
@@ -266,7 +261,7 @@ class Cursor(object):
     def get_current_line(self):
         keep_going = True
         i, it = self.index, self.parent.order[self.paragraph].get_iter()
-        ln = 0 
+        ln = 0
         while keep_going:
             l = it.get_line()
             ls = l.start_index
@@ -292,7 +287,7 @@ class Cursor(object):
                 return self.paragraph, (start, j)
 
             elif text[j] in self.WORD_TERMINATORS:
-                start = j+1
+                start = j + 1
 
             keep_going = it.next_char()
         return self.paragraph, (start, len(layout))
@@ -311,7 +306,7 @@ class PrimaryCursor(Cursor):
         Cursor.__init__(self, parent)
 
     def __repr__(self):
-        return 'Cursor: '+str((self.paragraph, self.index))
+        return 'Cursor: ' + str((self.paragraph, self.index))
 
     def get_rectangle(self, layout, a):
         if self.index < len(layout):
@@ -319,15 +314,14 @@ class PrimaryCursor(Cursor):
         else:
             pos = layout.get_cursor_pos(len(layout))[1]
 
-        x = layout.allocation.x + pos.x/_PS
-        y = layout.allocation.y + pos.y/_PS
-        return x, y, 1, pos.height/_PS
+        x = layout.allocation.x + pos.x / _PS
+        y = layout.allocation.y + pos.y / _PS
+        return x, y, 1, pos.height / _PS
 
     def draw(self, cr, layout, a):
-        cr.set_source_rgb(0,0,0)
+        cr.set_source_rgb(0, 0, 0)
         cr.rectangle(*self.get_rectangle(layout, a))
         cr.fill()
-        return
 
     def zero(self):
         self.index = 0
@@ -344,7 +338,7 @@ class SelectionCursor(Cursor):
         self.restore_point = None
 
     def __repr__(self):
-        return 'Selection: '+str(self.get_range())
+        return 'Selection: ' + str(self.get_range())
 
     def __nonzero__(self):
         c = self.cursor
@@ -372,7 +366,6 @@ class SelectionCursor(Cursor):
     def set_target_x(self, x, indent):
         self.target_x = x
         self.target_x_indent = indent
-        return
 
     def get_range(self):
         return self.min, self.max
@@ -401,10 +394,10 @@ class TextBlock(Gtk.EventBox):
         self.set_size_request(200, -1)
 
         self.set_can_focus(True)
-        self.set_events(Gdk.EventMask.KEY_PRESS_MASK|
-                        Gdk.EventMask.ENTER_NOTIFY_MASK|
-                        Gdk.EventMask.LEAVE_NOTIFY_MASK|
-                        Gdk.EventMask.BUTTON_RELEASE_MASK|
+        self.set_events(Gdk.EventMask.KEY_PRESS_MASK |
+                        Gdk.EventMask.ENTER_NOTIFY_MASK |
+                        Gdk.EventMask.LEAVE_NOTIFY_MASK |
+                        Gdk.EventMask.BUTTON_RELEASE_MASK |
                         Gdk.EventMask.POINTER_MOTION_MASK)
 
         self._is_new = False
@@ -431,7 +424,8 @@ class TextBlock(Gtk.EventBox):
         self.menu.append(self.select_all_menuitem)
         self.menu.show_all()
         self.copy_menuitem.connect('select', self._menu_do_copy, sel)
-        self.select_all_menuitem.connect('select', self._menu_do_select_all, cur, sel)
+        self.select_all_menuitem.connect('select', self._menu_do_select_all,
+            cur, sel)
 
         #~ Gtk.drag_source_set(self, Gdk.ModifierType.BUTTON1_MASK,
                             #~ None, Gdk.DragAction.COPY)
@@ -441,9 +435,12 @@ class TextBlock(Gtk.EventBox):
 
         event_helper = EventHelper()
 
-        self.connect('button-press-event', self._on_press, event_helper, cur, sel)
-        self.connect('button-release-event', self._on_release, event_helper, cur, sel) 
-        self.connect('motion-notify-event', self._on_motion, event_helper, cur, sel)
+        self.connect('button-press-event', self._on_press, event_helper, cur,
+            sel)
+        self.connect('button-release-event', self._on_release, event_helper,
+            cur, sel)
+        self.connect('motion-notify-event', self._on_motion, event_helper,
+            cur, sel)
         self.connect('key-press-event', self._on_key_press, cur, sel)
         self.connect('key-release-event', self._on_key_release, cur, sel)
         self.connect('focus-in-event', self._on_focus_in)
@@ -451,7 +448,6 @@ class TextBlock(Gtk.EventBox):
 
         self.connect("size-allocate", self.on_size_allocate)
         self.connect('style-updated', self._on_style_updated)
-        return
 
     def on_size_allocate(self, *args):
         allocation = self.get_allocation()
@@ -459,20 +455,20 @@ class TextBlock(Gtk.EventBox):
 
         x = y = 0
         for layout in self.order:
-            layout.set_width(_PS*(width-layout.indent))
+            layout.set_width(_PS * (width - layout.indent))
             if layout.index > 0:
                 y += (layout.vspacing or self.line_height)
 
             e = layout.get_pixel_extents()
             if self.get_direction() != Gtk.TextDirection.RTL:
-                layout.set_allocation(e.x+layout.indent, y+e.y,
-                                      width-layout.indent, e.height)
+                layout.set_allocation(e.x + layout.indent, y + e.y,
+                                      width - layout.indent, e.height)
             else:
-                layout.set_allocation(x+width-e.x-e.width-layout.indent-1, y+e.y,
-                                      width-layout.indent, e.height)
+                layout.set_allocation(x + width - e.x - e.width -
+                    layout.indent - 1, y + e.y, width - layout.indent,
+                    e.height)
 
             y += e.y + e.height
-        return
 
     # overrides
     def do_get_request_mode(self):
@@ -483,7 +479,7 @@ class TextBlock(Gtk.EventBox):
         layout = self._test_layout
         for l in self.order:
             layout.set_text(l.get_text(), -1)
-            layout.set_width(_PS*(width-l.indent))
+            layout.set_width(_PS * (width - l.indent))
             lh = layout.get_pixel_extents()[1].height
             height += lh + (l.vspacing or self.line_height)
 
@@ -492,7 +488,6 @@ class TextBlock(Gtk.EventBox):
 
     def do_draw(self, cr):
         self.render(self, cr)
-        return
 
     def _config_colors(self):
         context = self.get_style_context()
@@ -505,40 +500,37 @@ class TextBlock(Gtk.EventBox):
         self._bg = color_to_hex(context.get_background_color(state))
         self._fg = color_to_hex(context.get_color(state))
         context.restore()
-        return
 
     def _on_style_updated(self, widget):
         self._config_colors()
         self._update_cached_layouts()
-        return
 
 #    def _on_drag_begin(self, widgets, context, event_helper):
 #        print 'drag: begin'
-#        return
 
-    def _on_drag_data_get(self, widget, context, selection, info, timestamp, sel):
+    def _on_drag_data_get(self, widget, context, selection, info, timestamp,
+        sel):
 #        print 'drag: get data'
         text = self.get_selected_text(sel)
         selection.set_text(text, -1)
-        return
 
     def _on_focus_in(self, widget, event):
         self._config_colors()
-        return
 
     def _on_focus_out(self, widget, event):
         self._config_colors()
-        return
 
     def _on_motion(self, widget, event, event_helper, cur, sel):
 
-        if not (event.state == Gdk.ModifierType.BUTTON1_MASK):# or not self.has_focus():
+        if not (event.state == Gdk.ModifierType.BUTTON1_MASK):
+            # or not self.has_focus():
             return
 
         # check if we have moved enough to count as a drag
         press = event_helper['event']
         # mvo: how can this be?
-        if not press: return
+        if not press:
+            return
 
         start_x, start_y = int(press.x), int(press.y)
         cur_x, cur_y = int(event.x), int(event.y)
@@ -547,18 +539,18 @@ class TextBlock(Gtk.EventBox):
             self.drag_check_threshold(start_x, start_y, cur_x, cur_y)):
             event_helper['drag-active'] = True
 
-        if not event_helper['drag-active']: 
+        if not event_helper['drag-active']:
             return
 
-        #~ if (event_helper['within-selection'] and 
+        #~ if (event_helper['within-selection'] and
             #~ not event_helper['drag-context']):
             #~ target_list = Gtk.TargetList()
             #~ target_list.add_text_targets(80)
-            #~ ctx = self.drag_begin(target_list,              # target list
-                                  #~ Gdk.DragAction.COPY,      # action
-                                  #~ 1,                        # initiating button
-                                  #~ event)                    # event
-#~ 
+            #~ ctx = self.drag_begin(target_list,           # target list
+                                  #~ Gdk.DragAction.COPY,   # action
+                                  #~ 1,                     # initiating button
+                                  #~ event)                 # event
+#~
             #~ event_helper['drag-context'] = ctx
             #~ return
 
@@ -596,13 +588,14 @@ class TextBlock(Gtk.EventBox):
                     cur.set_position(layout.index, index)
                     sel.clear()
 
-				#~ event_helper.new_press(event.copy(), layout, index, within_sel)
+                #~ event_helper.new_press(event.copy(), layout, index,
+                #~     within_sel)
                 event_helper.new_press(event, layout, index, within_sel)
                 break
-        return
 
     def _on_release(self, widget, event, event_helper, cur, sel):
-        if not event_helper['event']: return
+        if not event_helper['event']:
+            return
 
         # check if a drag occurred
         if event_helper['drag-active']:
@@ -622,7 +615,6 @@ class TextBlock(Gtk.EventBox):
             self._3click_select(cur, sel)
 
         self.queue_draw()
-        return
 
     def _menu_do_copy(self, item, sel):
         self._copy_text(sel)
@@ -639,16 +631,15 @@ class TextBlock(Gtk.EventBox):
         if not sel:
             self.copy_menuitem.set_sensitive(False)
         elif start == (0, 0) and \
-            end == (len(self.order)-1, len(self.order[-1])):
+            end == (len(self.order) - 1, len(self.order[-1])):
             self.select_all_menuitem.set_sensitive(False)
 
-        self.menu.popup(None, # parent_menu_shell,
-                        None, # parent_menu_item,
-                        None, # GtkMenuPositionFunc func,
-                        None, # data,
+        self.menu.popup(None,  # parent_menu_shell,
+                        None,  # parent_menu_item,
+                        None,  # GtkMenuPositionFunc func,
+                        None,  # data,
                         event.button,
                         event.time)
-        return
 
     def _on_key_press(self, widget, event, cur, sel):
         kv = event.keyval
@@ -676,7 +667,7 @@ class TextBlock(Gtk.EventBox):
                 pos = layout.index_to_pos(cur.index)
                 sel.set_target_x(pos.x, layout.indent)
 
-        elif kv == Gdk.KEY_Right: 
+        elif kv == Gdk.KEY_Right:
             if ctrl:
                 self._select_right_word(cur, sel, s, i)
             else:
@@ -701,7 +692,7 @@ class TextBlock(Gtk.EventBox):
         elif kv == Gdk.KEY_Down:
             if ctrl:
                 if i == len(self._get_layout(cur)):
-                    if s+1 < len(self.order):
+                    if s + 1 < len(self.order):
                         cur.paragraph += 1
                 i = len(self._get_layout(cur))
                 cur.set_position(cur.paragraph, i)
@@ -720,7 +711,7 @@ class TextBlock(Gtk.EventBox):
             if shift:
                 self._select_end(cur, sel, self.order[cur.paragraph])
             else:
-                cur.paragraph = len(self.order)-1
+                cur.paragraph = len(self.order) - 1
                 cur.index = len(self._get_layout(cur))
 
         else:
@@ -742,7 +733,6 @@ class TextBlock(Gtk.EventBox):
                 self._copy_text(sel)
 
             self.queue_draw()
-        return
 
     def _select_up(self, cur, sel):
         #~ if sel and not cur.is_min(sel) and cur.same_line(sel):
@@ -767,13 +757,13 @@ class TextBlock(Gtk.EventBox):
             cur.set_position(s, j)
 
         elif s > 0:
-            cur.paragraph = s-1
+            cur.paragraph = s - 1
             layout = self._get_layout(cur)
             if sel.target_x_indent:
                 x += (sel.target_x_indent - layout.indent) * _PS
             y = layout.get_extents()[0].height
             (_, j, k) = layout.xy_to_index(x, y)
-            cur.set_position(s-1, j+k)
+            cur.set_position(s - 1, j + k)
 
         else:
             return False
@@ -802,13 +792,13 @@ class TextBlock(Gtk.EventBox):
             cur.set_position(s, j)
 
         elif s < len(self.order) - 1:
-            cur.paragraph = s+1
+            cur.paragraph = s + 1
             layout = self._get_layout(cur)
             if sel.target_x_indent:
                 x += (sel.target_x_indent - layout.indent) * _PS
             y = 0
             (_, j, k) = layout.xy_to_index(x, y)
-            cur.set_position(s+1, j+k)
+            cur.set_position(s + 1, j + k)
 
         else:
             return False
@@ -816,7 +806,6 @@ class TextBlock(Gtk.EventBox):
 
     def _2click_select(self, cursor, sel):
         self._select_word(cursor, sel)
-        return
 
     def _3click_select(self, cursor, sel):
         # XXX:
@@ -827,7 +816,6 @@ class TextBlock(Gtk.EventBox):
         # ... which can result in a segfault
         #~ self._select_line(cursor, sel)
         self._select_all(cursor, sel)
-        return
 
     def _copy_text(self, sel):
 
@@ -840,7 +828,6 @@ class TextBlock(Gtk.EventBox):
 
         self.clipboard.clear()
         self.clipboard.set_text(text.strip(), -1)
-        return
 
     def _select_end(self, cur, sel, layout):
         if not cur.is_max(sel):
@@ -849,7 +836,7 @@ class TextBlock(Gtk.EventBox):
         n, r, line = cur.get_current_line()
         cur_pos = cur.get_position()
 
-        if cur_pos == (len(self.order)-1, len(self.order[-1])):   # absolute end
+        if cur_pos == (len(self.order) - 1, len(self.order[-1])):  # abs end
             if sel.restore_point:
                 # reinstate restore point
                 cur.set_position(*sel.restore_point)
@@ -860,7 +847,7 @@ class TextBlock(Gtk.EventBox):
 
         elif cur_pos[1] == len(self.order[n[0]]):   # para end
             # select abs end
-            cur.set_position(len(self.order)-1, len(self.order[-1]))
+            cur.set_position(len(self.order) - 1, len(self.order[-1]))
 
         elif cur_pos == (n[0], r[1]):   # line end
             # select para end
@@ -871,7 +858,6 @@ class TextBlock(Gtk.EventBox):
             if sel:
                 sel.restore_point = cur_pos
             cur.set_position(n[0], r[1])
-        return
 
     def _select_home(self, cur, sel, layout):
         if not cur.is_min(sel):
@@ -888,7 +874,7 @@ class TextBlock(Gtk.EventBox):
                 cur.set_position(n[0], r[0])
 
         elif cur_pos[1] == 0:   # para home
-            cur.set_position(0,0)
+            cur.set_position(0, 0)
 
         elif cur_pos == (n[0], r[0]):      # line home
             cur.set_position(n[0], 0)
@@ -897,28 +883,25 @@ class TextBlock(Gtk.EventBox):
             if sel:
                 sel.restore_point = cur_pos
             cur.set_position(n[0], r[0])
-        return
 
     def _select_left(self, cur, sel, s, i, shift):
         if not shift and not cur.is_min(sel):
             cur.switch(sel)
             return
         if i > 0:
-            cur.set_position(s, i-1)
+            cur.set_position(s, i - 1)
         elif cur.paragraph > 0:
             cur.paragraph -= 1
-            cur.set_position(s-1, len(self._get_layout(cur)))
-        return
+            cur.set_position(s - 1, len(self._get_layout(cur)))
 
     def _select_right(self, cur, sel, s, i, shift):
         if not shift and not cur.is_max(sel):
             cur.switch(sel)
             return
         if i < len(self._get_layout(cur)):
-            cur.set_position(s, i+1)
-        elif s < len(self.order)-1:
-            cur.set_position(s+1, 0)
-        return
+            cur.set_position(s, i + 1)
+        elif s < len(self.order) - 1:
+            cur.set_position(s + 1, 0)
 
     def _select_left_word(self, cur, sel, s, i):
         if i > 0:
@@ -928,31 +911,30 @@ class TextBlock(Gtk.EventBox):
             cur.index = len(self._get_layout(cur))
 
         paragraph, word = cur.get_current_word()
-        if not word: return
-        cur.set_position(paragraph, max(0, word[0]-1))
-        return
+        if not word:
+            return
+        cur.set_position(paragraph, max(0, word[0] - 1))
 
     def _select_right_word(self, cur, sel, s, i):
         ll = len(self._get_layout(cur))
         if i < ll:
             cur.index += 1
-        elif s+1 < len(self.order):
+        elif s + 1 < len(self.order):
             cur.paragraph += 1
             cur.index = 0
 
         paragraph, word = cur.get_current_word()
-        if not word: return
-        cur.set_position(paragraph, min(word[1]+1, ll))
-        return
+        if not word:
+            return
+        cur.set_position(paragraph, min(word[1] + 1, ll))
 
     def _select_word(self, cursor, sel):
         paragraph, word = cursor.get_current_word()
         if word:
-            cursor.set_position(paragraph, word[1]+1)
+            cursor.set_position(paragraph, word[1] + 1)
             sel.set_position(paragraph, word[0])
             if self.get_direction() == Gtk.TextDirection.RTL:
                 cursor.switch(sel)
-        return
 
     def _select_line(self, cursor, sel):
         n, r = self.cursor.get_current_line()
@@ -960,7 +942,6 @@ class TextBlock(Gtk.EventBox):
         cursor.set_position(n[0], r[1])
         if self.get_direction() == Gtk.TextDirection.RTL:
             cursor.switch(sel)
-        return
 
     def _select_all(self, cursor, sel):
         layout = self.order[-1]
@@ -968,7 +949,6 @@ class TextBlock(Gtk.EventBox):
         cursor.set_position(layout.index, len(layout))
         if self.get_direction() == Gtk.TextDirection.RTL:
             cursor.switch(sel)
-        return
 
     def _selection_copy(self, layout, sel, new_para=True):
         i = layout.index
@@ -983,18 +963,18 @@ class TextBlock(Gtk.EventBox):
 
             if i == start[0]:
                 if end[0] > i:
-                    return text+layout.get_text()[start[1]: len(layout)]
+                    return text + layout.get_text()[start[1]: len(layout)]
                 else:
-                    return text+layout.get_text()[start[1]: end[1]]
+                    return text + layout.get_text()[start[1]: end[1]]
 
             elif i == end[0]:
                 if start[0] < i:
-                    return text+layout.get_text()[0: end[1]]
+                    return text + layout.get_text()[0: end[1]]
                 else:
-                    return text+layout.get_text()[start[1]: end[1]]
+                    return text + layout.get_text()[start[1]: end[1]]
 
             else:
-                return text+layout.get_text()
+                return text + layout.get_text()
         return ''
 
     def _new_layout(self, text=''):
@@ -1011,8 +991,7 @@ class TextBlock(Gtk.EventBox):
         self._bullet.set_font_description(font_desc)
 
         e = self._bullet.get_pixel_extents()
-        self.indent, self.line_height  = e.width, e.height
-        return
+        self.indent, self.line_height = e.width, e.height
 
     def _selection_highlight(self, layout, sel, bg, fg):
         i = layout.index
@@ -1035,7 +1014,6 @@ class TextBlock(Gtk.EventBox):
 
         elif not layout._default_attrs:
             layout.reset_attrs()
-        return
 
     def _paint_bullet_point(self, cr, x, y):
         # draw the layout
@@ -1055,7 +1033,7 @@ class TextBlock(Gtk.EventBox):
         return self.order[self.selection.paragraph]
 
     def render(self, widget, cr):
-        if not self.order: 
+        if not self.order:
             return
 
         a = self.get_allocation()
@@ -1076,7 +1054,7 @@ class TextBlock(Gtk.EventBox):
             if self.DEBUG_PAINT_BBOXES:
                 la = layout.allocation
                 cr.rectangle(la.x, la.y, la.width, la.height)
-                cr.set_source_rgb(1,0,0)
+                cr.set_source_rgb(1, 0, 0)
                 cr.stroke()
 
             # draw the layout
@@ -1089,7 +1067,6 @@ class TextBlock(Gtk.EventBox):
         # draw the cursor
         if self.PAINT_PRIMARY_CURSOR and self.has_focus():
             self.cursor.draw(cr, self._get_layout(self.cursor), a)
-        return
 
     def append_paragraph(self, p, vspacing=None):
         l = self._new_layout()
@@ -1097,7 +1074,6 @@ class TextBlock(Gtk.EventBox):
         l.vspacing = vspacing
         l.set_text(p)
         self.order.append(l)
-        return
 
     def append_bullet(self, point, indent_level, vspacing=None):
         l = self._new_layout()
@@ -1108,11 +1084,9 @@ class TextBlock(Gtk.EventBox):
 
         l.set_text(point)
         self.order.append(l)
-        return
 
     def copy_clipboard(self):
         self._copy_text(self.selection)
-        return
 
     def get_selected_text(self, sel=None):
         text = ''
@@ -1125,23 +1099,20 @@ class TextBlock(Gtk.EventBox):
     def select_all(self):
         self._select_all(self.cursor, self.selection)
         self.queue_draw()
-        return
 
     def finished(self):
         self.queue_resize()
-        return
 
     def clear(self, key=None):
         self.cursor.zero()
         self.selection.clear(key)
         self.order = []
-        return
 
 
 class AppDescription(Gtk.VBox):
 
     TYPE_PARAGRAPH = 0
-    TYPE_BULLET    = 1
+    TYPE_BULLET = 1
 
     _preparser = _SpecialCasePreParsers()
 
@@ -1150,7 +1121,6 @@ class AppDescription(Gtk.VBox):
         self.description = TextBlock()
         self.pack_start(self.description, False, False, 0)
         self._prev_type = None
-        return
 
     def _part_is_bullet(self, part):
         # normalize_description() ensures that we only have "* " bullets
@@ -1158,8 +1128,8 @@ class AppDescription(Gtk.VBox):
         return i > -1, i
 
     def _parse_desc(self, desc, pkgname):
-        """ Attempt to maintain original fixed width layout, while 
-            reconstructing the description into text blocks 
+        """ Attempt to maintain original fixed width layout, while
+            reconstructing the description into text blocks
             (either paragraphs or bullets) which are line-wrap friendly.
         """
         # pre-parse descrition if special case exists for the given pkgname
@@ -1167,7 +1137,8 @@ class AppDescription(Gtk.VBox):
 
         parts = normalize_package_description(desc).split('\n')
         for part in parts:
-            if not part: continue
+            if not part:
+                continue
             is_bullet, indent = self._part_is_bullet(part)
             if is_bullet:
                 self.append_bullet(part, indent)
@@ -1175,28 +1146,24 @@ class AppDescription(Gtk.VBox):
                 self.append_paragraph(part)
 
         self.description.finished()
-        return
 
     def clear(self):
         self.description.clear()
-        return
 
     def append_paragraph(self, p):
         vspacing = self.description.line_height
         self.description.append_paragraph(p.strip(), vspacing)
         self._prev_type = self.TYPE_PARAGRAPH
-        return
 
     def append_bullet(self, point, indent_level):
         if self._prev_type == self.TYPE_BULLET:
-            vspacing = int(0.4*self.description.line_height)
+            vspacing = int(0.4 * self.description.line_height)
         else:
             vspacing = self.description.line_height
 
         self.description.append_bullet(
-                        point[indent_level+2:], indent_level, vspacing)
+                        point[indent_level + 2:], indent_level, vspacing)
         self._prev_type = self.TYPE_BULLET
-        return
 
     def set_description(self, raw_desc, pkgname):
         self.clear()
@@ -1204,10 +1171,9 @@ class AppDescription(Gtk.VBox):
             encoded_desc = unicode(raw_desc, 'utf8').encode('utf8')
         else:
             encoded_desc = raw_desc.encode('utf8')
-        desc = GObject.markup_escape_text(encoded_desc)
-        self._parse_desc(desc, pkgname)
+        self._text = GObject.markup_escape_text(encoded_desc)
+        self._parse_desc(self._text, pkgname)
         self.show_all()
-        return
 
     # easy access to some TextBlock methods
     def copy_clipboard(self):
@@ -1221,21 +1187,29 @@ class AppDescription(Gtk.VBox):
 
 
 def get_test_description_window():
-    EXAMPLE0 = """
-p7zip is the Unix port of 7-Zip, a file archiver that archives with very high compression ratios.
+    EXAMPLE0 = """p7zip is the Unix port of 7-Zip, a file archiver that \
+archives with very high compression ratios.
 
 p7zip-full provides:
 
- - /usr/bin/7za a standalone version of the 7-zip tool that handles 
-   7z archives (implementation of the LZMA compression algorithm) and some other formats.
+ - /usr/bin/7za a standalone version of the 7-zip tool that handles
+   7z archives (implementation of the LZMA compression algorithm) and some \
+other formats.
 
- - /usr/bin/7z not only does it handle 7z but also ZIP, Zip64, CAB, RAR, ARJ, GZIP, 
-   BZIP2, TAR, CPIO, RPM, ISO and DEB archives. 7z compression is 30-50% better than ZIP compression.
+ - /usr/bin/7z not only does it handle 7z but also ZIP, Zip64, CAB, RAR, \
+ARJ, GZIP,
+   BZIP2, TAR, CPIO, RPM, ISO and DEB archives. 7z compression is 30-50% \
+better than ZIP compression.
 
-p7zip provides 7zr, a light version of 7za, and p7zip a gzip like wrapper around 7zr.""".strip()
+p7zip provides 7zr, a light version of 7za, and p7zip a gzip like wrapper \
+around 7zr."""
 
-
-    EXAMPLE1 = """Transmageddon supports almost any format as its input and can generate a very large host of output files. The goal of the application was to help people to create the files they need to be able to play on their mobile devices and for people not hugely experienced with multimedia to generate a multimedia file without having to resort to command line tools with ungainly syntaxes.
+    EXAMPLE1 = """Transmageddon supports almost any format as its input and \
+can generate a very large host of output files. The goal of the application \
+was to help people to create the files they need to be able to play on their \
+mobile devices and for people not hugely experienced with multimedia to \
+generate a multimedia file without having to resort to command line tools \
+with ungainly syntaxes.
 The currently supported codecs are:
  * Containers:
   - Ogg
@@ -1265,8 +1239,8 @@ The currently supported codecs are:
   - DNxHD
 It also provide the support for the GStreamer's plugins auto-search."""
 
-
-    EXAMPLE2 = """File-roller is an archive manager for the GNOME environment. It allows you to:
+    EXAMPLE2 = """File-roller is an archive manager for the GNOME \
+environment. It allows you to:
  * Create and modify archives.
  * View the content of an archive.
  * View a file contained in an archive.
@@ -1283,7 +1257,8 @@ File-roller supports the following formats:
  * Lha archives (.lzh)
  * Single files compressed with gzip (.gz), bzip (.bz), bzip2 (.bz2),
    compress (.Z), lzip (.lz), lzop (.lzo), lzma (.lzma) and xz (.xz)
-File-roller doesn't perform archive operations by itself, but relies on standard tools for this."""
+File-roller doesn't perform archive operations by itself, but relies on \
+standard tools for this."""
 
     EXAMPLE3 = """This package includes the following CTAN packages:
  Asana-Math -- A font to typeset maths in Xe(La)TeX.
@@ -1422,9 +1397,11 @@ File-roller doesn't perform archive operations by itself, but relies on standard
  yfonts -- Support for old German fonts.
  zefonts -- Virtual fonts to provide T1 encoding from existing fonts."""
 
-
-    EXAMPLE4 = """Arista is a simple multimedia transcoder, it focuses on being easy to use by making complex task of encoding for various devices simple.
-Users should pick an input and a target device, choose a file to save to and go. Features:
+    EXAMPLE4 = """Arista is a simple multimedia transcoder, it focuses on \
+being easy to use by making complex task of encoding for various devices \
+simple.
+Users should pick an input and a target device, choose a file to save to and \
+go. Features:
 * Presets for iPod, computer, DVD player, PSP, Playstation 3, and more.
 * Live preview to see encoded quality.
 * Automatically discover available DVD media and Video 4 Linux (v4l) devices.
@@ -1438,13 +1415,12 @@ Users should pick an input and a target device, choose a file to save to and go.
         if widget.position >= len(descs):
             widget.position = 0
         desc_widget.set_description(*descs[widget.position])
-        return
 
-    descs = ((EXAMPLE0,''),
-             (EXAMPLE1,''),
-             (EXAMPLE2,''),
-             (EXAMPLE3,'texlive-fonts-extra'),
-             (EXAMPLE4,''))
+    descs = ((EXAMPLE0, ''),
+             (EXAMPLE1, ''),
+             (EXAMPLE2, ''),
+             (EXAMPLE3, 'texlive-fonts-extra'),
+             (EXAMPLE4, ''))
 
     win = Gtk.Window()
     win.set_default_size(300, 400)
@@ -1463,11 +1439,10 @@ Users should pick an input and a target device, choose a file to save to and go.
     win.show_all()
 
     b.connect("clicked", on_clicked, d, descs)
-    win.connect('destroy',lambda x: Gtk.main_quit())
+    win.connect('destroy', lambda x: Gtk.main_quit())
     return win
 
 if __name__ == '__main__':
     win = get_test_description_window()
     win.show_all()
     Gtk.main()
-
