@@ -1,17 +1,12 @@
 #!/usr/bin/python
 
 from gi.repository import Gtk
-import sys
 import time
 import unittest
 
-sys.path.insert(0,"../..")
-sys.path.insert(0,"..")
+from testutils import setup_test_env
+setup_test_env()
 
-#from mock import Mock
-
-import softwarecenter.paths
-softwarecenter.paths.datadir = "../data"
 
 from softwarecenter.db.application import Application
 from softwarecenter.testutils import start_dummy_backend, stop_dummy_backend
@@ -27,13 +22,13 @@ class TestViews(unittest.TestCase):
         stop_dummy_backend()
 
     def test_install_appdetails(self):
-        from softwarecenter.ui.gtk3.views.appdetailsview_gtk import get_test_window_appdetails
+        from softwarecenter.ui.gtk3.views.appdetailsview import get_test_window_appdetails
         win = get_test_window_appdetails()
         view = win.get_data("view")
         view.show_app(Application("", "2vcard"))
         self._p()
         app = view.app
-        view.backend.install(app.pkgname, app.appname, "")
+        view.backend.install(app, "")
         self._p()
         self.assertTrue(view.pkg_statusbar.progress.get_property("visible"))
 
