@@ -1,17 +1,16 @@
 #!/usr/bin/python
 
-import sys
-sys.path.insert(0,"../")
-
 import logging
 import unittest
 
+from testutils import setup_test_env
+setup_test_env()
 from softwarecenter.plugin import PluginManager
 
 class MockApp(object):
     """ mock app """
 
-class testPlugin(unittest.TestCase):
+class TestPlugin(unittest.TestCase):
 
     def setUp(self):
         pass
