@@ -1,17 +1,15 @@
 #!/usr/bin/python
 
 import os
-import sys
 import time
 import unittest
 
 from gi.repository import GObject, Gtk
 from mock import Mock
 
-sys.path.insert(0, "..")
-
+from testutils import setup_test_env
+setup_test_env()
 from softwarecenter.db.application import Application
-
 from softwarecenter.backend import get_install_backend
 
 #import softwarecenter.log
@@ -51,6 +49,7 @@ class TestPurchaseBackend(unittest.TestCase):
 		while Gtk.events_pending():
 			Gtk.main_iteration()
 		time.sleep(0.1)
+        if os.getuid() == 0:
             self.assertTrue(os.path.exists(self.LICENSE_KEY_PATH))
             self.assertEqual(open(self.LICENSE_KEY_PATH).read(), self.LICENSE_KEY)
         #time.sleep(10)
